# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Pin definitions for the Rock Pi E."""

from adafruit_blinka.microcontroller.rockchip.rk3328 import pin

D3 = pin.GPIO3_A4  # /UART1_TX/PIN 100/
D5 = pin.GPIO3_A6  # /UART1_RX/PIN 102/
D7 = pin.GPIO1_D4  # /PIN 60/
D8 = pin.GPIO2_A0  # /UART2_TX/PIN 64/
D10 = pin.GPIO2_A1  # /UART2_RX/PIN 65/
D11 = pin.GPIO2_A2  # /PWM_IR/PIN 66/
D12 = pin.GPIO2_C2  # /PDM_CLK/I2S1_SCLK/PIN 82/
D13 = pin.GPIO2_A3  # /PIN 67/
D15 = pin.GPIO0_D3  # /PIN 27/
D19 = pin.GPIO3_A1  # /SPI0_TXD/PIN 97/
D21 = pin.GPIO3_A2  # /SPI0_RXD/PIN 98/
D22 = pin.GPIO2_B4
D23 = pin.GPIO3_A0  # /SPI0_CLK/PIN 96/
D24 = pin.GPIO3_B0  # /SPI0_CSN0/PIN 104/
D26 = pin.GPIO2_B4  # /PIN 76/
D27 = pin.GPIO2_A4  # /I2C1_SDAPIN 68/
D28 = pin.GPIO2_A5  # /I2C1_SCL/PIN 69/
D29 = pin.GPIO2_C4  # /I2S1_SDIO1/PDM_SDI1/PIN 84/
D31 = pin.GPIO2_C5  # /I2S1_SDIO2/PDM_SDI2/PIN 85/
D32 = pin.GPIO2_C0  # /I2S1_LRCK_RX/PIN 80/
D33 = pin.GPIO2_A6  # /PWM2/PIN 70/
D35 = pin.GPIO2_C1  # /I2S1_LRCK_TX/PIN 81/
D36 = pin.GPIO2_B7  # /I2S1_MCLK/PIN 79/
D37 = pin.GPIO2_C6  # /PDM_SDI3/PIN 86/
D38 = pin.GPIO2_C3  # /PDM_SDI0/I2S1_SDI/PIN 83/
D40 = pin.GPIO2_C7  # /PDM_FSYNC/I2S1_SDO/PIN 87/

SDA2 = D27
SCL2 = D28

SDA = SDA2
SCL = SCL2

SCLK = D23
MOSI = D19
MISO = D21
CS = D24
SCK = SCLK

UART1_TX = D3
UART2_RX = D5
UART2_TX = D8
UART2_RX = D10

UART4_TX = D19
UART4_RX = D21

UART_TX = UART2_TX
UART_RX = UART2_RX

PWM2 = pin.PWM2



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""A Pin class for use with libgpiod."""
try:
    import gpiod
except ImportError:
    raise ImportError(
        "libgpiod Python bindings not found, please install and try again! See "
        "https://github.com/adafruit/Raspberry-Pi-Installer-Scripts/blob/master/libgpiod.sh"
    ) from ImportError

# pylint: disable=too-many-branches,too-many-statements
class Pin:
    """Pins dont exist in CPython so...lets make our own!"""

    IN = 0
    OUT = 1
    LOW = 0
    HIGH = 1
    PULL_NONE = 0
    PULL_UP = 1
    PULL_DOWN = 2
    _CONSUMER = "adafruit_blinka"

    id = None
    _value = LOW
    _mode = IN

    def __init__(self, pin_id):
        self.id = pin_id
        if isinstance(pin_id, tuple):
            self._num = int(pin_id[1])
            if hasattr(gpiod, "Chip"):
                self._chip = gpiod.Chip(str(pin_id[0]), gpiod.Chip.OPEN_BY_NUMBER)
            else:
                self._chip = gpiod.chip(str(pin_id[0]), gpiod.chip.OPEN_BY_NUMBER)
        else:
            self._num = int(pin_id)
            if hasattr(gpiod, "Chip"):
                self._chip = gpiod.Chip("gpiochip0", gpiod.Chip.OPEN_BY_NAME)
            else:
                self._chip = gpiod.chip("gpiochip0", gpiod.chip.OPEN_BY_NAME)
        self._line = None

    def __repr__(self):
        return str(self.id)

    def __eq__(self, other):
        return self.id == other

    def init(self, mode=IN, pull=None):
        """Initialize the Pin"""
        if not self._line:
            self._line = self._chip.get_line(int(self._num))
            # print("init line: ", self.id, self._line)

        if mode is not None:
            if mode == self.IN:
                flags = 0
                self._line.release()
                if pull is not None:
                    if pull == self.PULL_UP:
                        if hasattr(gpiod, "line") and hasattr(
                            gpiod.line, "BIAS_PULL_UP"
                        ):
                            config = gpiod.line_request()
                            config.consumer = self._CONSUMER
                            config.request_type = gpiod.line.BIAS_PULL_UP
                            self._line.request(config)
                        else:
                            self._line.request(
                                consumer=self._CONSUMER,
                                type=gpiod.LINE_REQ_DIR_IN,
                                flags=flags,
                            )
                            raise NotImplementedError(
                                "Internal pullups not supported in this version of libgpiod, "
                                "use physical resistor instead!"
                            )
                    elif pull == self.PULL_DOWN:
                        if hasattr(gpiod, "line") and hasattr(
                            gpiod.line, "BIAS_PULL_DOWN"
                        ):
                            config = gpiod.line_request()
                            config.consumer = self._CONSUMER
                            config.request_type = gpiod.line.BIAS_PULL_DOWN
                            self._line.request(config)
                        else:
                            raise NotImplementedError(
                                "Internal pulldowns not supported in this version of libgpiod, "
                                "use physical resistor instead!"
                            )
                    elif pull == self.PULL_NONE:
                        if hasattr(gpiod, "line") and hasattr(
                            gpiod.line, "BIAS_DISABLE"
                        ):
                            config = gpiod.line_request()
                            config.consumer = self._CONSUMER
                            config.request_type = gpiod.line.BIAS_DISABLE
                            self._line.request(config)
                    else:
                        raise RuntimeError(f"Invalid pull for pin: {self.id}")

                self._mode = self.IN
                self._line.release()
                if hasattr(gpiod, "LINE_REQ_DIR_IN"):
                    self._line.request(
                        consumer=self._CONSUMER, type=gpiod.LINE_REQ_DIR_IN, flags=flags
                    )
                else:
                    config = gpiod.line_request()
                    config.consumer = self._CONSUMER
                    config.request_type = gpiod.line_request.DIRECTION_INPUT
                    self._line.request(config)

            elif mode == self.OUT:
                if pull is not None:
                    raise RuntimeError("Cannot set pull resistor on output")
                self._mode = self.OUT
                self._line.release()
                if hasattr(gpiod, "LINE_REQ_DIR_OUT"):
                    self._line.request(
                        consumer=self._CONSUMER, type=gpiod.LINE_REQ_DIR_OUT
                    )
                else:
                    config = gpiod.line_request()
                    config.consumer = self._CONSUMER
                    config.request_type = gpiod.line_request.DIRECTION_OUTPUT
                    self._line.request(config)
            else:
                raise RuntimeError("Invalid mode for pin: %s" % self.id)

    def value(self, val=None):
        """Set or return the Pin Value"""
        if val is None:
            return self._line.get_value()

        if val in (self.LOW, self.HIGH):
            self._value = val
            self._line.set_value(val)
            return None
        raise RuntimeError("Invalid value for pin")



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""
Pin definitions for the ItsyBitsy RP2040 with u2if firmware.

Adafruit CircuitPython 6.2.0 on 2021-04-05; Adafruit ItsyBitsy RP2040 with rp2040
>>> import board
>>> board.
A0              A1              A2              A3
BUTTON          D0              D1              D10
D11             D12             D13             D2
D24             D25             D3              D4
D5              D7              D9              I2C
LED             MISO            MOSI            NEOPIXEL
NEOPIXEL_POWER  RX              SCK             SCL
SDA             SPI             TX              UART
"""


from adafruit_blinka.microcontroller.rp2040_u2if import pin

D0 = pin.GP1
D1 = pin.GP0
D2 = pin.GP12
D3 = pin.GP5
D4 = pin.GP4
D5 = pin.GP14
D7 = pin.GP6
D9 = pin.GP7
D10 = pin.GP8
D11 = pin.GP9
D12 = pin.GP10
D13 = pin.GP11
D24 = pin.GP24
D25 = pin.GP25

A0 = pin.GP26
A1 = pin.GP27
A2 = pin.GP28
# A3 = pin.GP29 # not currently supported in firmware

SCL = pin.GP3
SDA = pin.GP2

SCLK = SCK = pin.GP18
MOSI = pin.GP19
MISO = pin.GP20

NEOPIXEL = pin.GP17
NEOPIXEL_POWER = pin.GP16

BUTTON = pin.GP13

# access u2if via pin instance to open for specifc VID/PID
# pylint:disable = protected-access
pin.GP0._u2if_open_hid(0x239A, 0x00FD)



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""
Much code from https://github.com/vsergeev/python-periphery/blob/master/periphery/pwm.py
Copyright (c) 2015-2016 vsergeev / Ivan (Vanya) A. Sergeev
License: MIT
"""

import os
from time import sleep
from errno import EACCES

try:
    from microcontroller.pin import pwmOuts
except ImportError:
    raise RuntimeError("No PWM outputs defined for this board.") from ImportError


# pylint: disable=unnecessary-pass, too-many-instance-attributes


class PWMError(IOError):
    """Base class for PWM errors."""

    pass


# pylint: enable=unnecessary-pass


class PWMOut:
    """Pulse Width Modulation Output Class"""

    # Number of retries to check for successful PWM export on open
    PWM_STAT_RETRIES = 10
    # Delay between check for successful PWM export on open (100ms)
    PWM_STAT_DELAY = 0.1

    # Sysfs paths
    _chip_path = "pwmchip{}"
    _channel_path = "pwm{}"

    def __init__(self, pwm, *, frequency=500, duty_cycle=0, variable_frequency=False):
        """Instantiate a PWM object and open the sysfs PWM corresponding to the
        specified chip and channel.
        Args:
            pwm (str): PWM pin.
            frequency (int, float): target frequency in Hertz (32-bit).
            duty_cycle (int, float): The fraction of each pulse which is high (16-bit).
            variable_frequency (bool): True if the frequency will change over time.
        Returns:
            PWM: PWM object.
        Raises:
            PWMError: if an I/O or OS error occurs.
            TypeError: if `chip` or `channel` types are invalid.
            LookupError: if PWM chip does not exist.
            TimeoutError: if waiting for PWM export times out.
        """

        self._chip = None
        self._channel = None
        self._period_ns = 0
        self._open(pwm, frequency, duty_cycle, variable_frequency)

    def __del__(self):
        self.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def _open(self, pwm, frequency, duty_cycle, variable_frequency):
        for pwmout in pwmOuts:
            if pwmout[1] == pwm:
                self._chip = pwmout[0][0]
                self._channel = pwmout[0][1]

        self._chip_path = os.path.join(
            "/sys/class/pwm", self._chip_path.format(self._chip)
        )
        self._channel_path = os.path.join(
            self._chip_path, self._channel_path.format(self._channel)
        )

        if variable_frequency:
            print("Variable Frequency is not supported, continuing without it...")

        if not os.path.isdir(self._chip_path):
            raise LookupError("Opening PWM: PWM chip {} not found.".format(self._chip))

        if not os.path.isdir(self._channel_path):
            # Exporting the PWM.
            try:
                with open(
                    os.path.join(self._chip_path, "export"), "w", encoding="utf-8"
                ) as f_export:
                    f_export.write("{:d}\n".format(self._channel))
            except IOError as e:
                raise PWMError(
                    e.errno, "Exporting PWM channel: " + e.strerror
                ) from IOError

            # Loop until PWM is exported
            exported = False
            for i in range(PWMOut.PWM_STAT_RETRIES):
                if os.path.isdir(self._channel_path):
                    exported = True
                    break

                sleep(PWMOut.PWM_STAT_DELAY)

            if not exported:
                raise TimeoutError(
                    'Exporting PWM: waiting for "{:s}" timed out.'.format(
                        self._channel_path
                    )
                )

            # Loop until 'period' is writable, This could take some time after
            # export as application of the udev rules after export is asynchronous.
            # Without this loop, the following properties may not be writable yet.
            for i in range(PWMOut.PWM_STAT_RETRIES):
                try:
                    with open(
                        os.path.join(self._channel_path, "period"),
                        "w",
                        encoding="utf-8",
                    ):
                        break
                except IOError as e:
                    if e.errno != EACCES or (
                        e.errno == EACCES and i == PWMOut.PWM_STAT_RETRIES - 1
                    ):
                        raise PWMError(
                            e.errno, "Opening PWM period: " + e.strerror
                        ) from IOError

                sleep(PWMOut.PWM_STAT_DELAY)

            self.frequency = frequency
            self.duty_cycle = duty_cycle

            # Cache the period for fast duty cycle updates
            self._period_ns = self._get_period_ns()

    def close(self):
        """Close the PWM."""
        if self._channel is not None:
            # Unexporting the PWM channel
            try:
                unexport_fd = os.open(
                    os.path.join(self._chip_path, "unexport"), os.O_WRONLY
                )
                os.write(unexport_fd, "{:d}\n".format(self._channel).encode())
                os.close(unexport_fd)
            except OSError as e:
                raise PWMError(e.errno, "Unexporting PWM: " + e.strerror) from OSError

        self._chip = None
        self._channel = None

    def _write_channel_attr(self, attr, value):
        with open(
            os.path.join(self._channel_path, attr), "w", encoding="utf-8"
        ) as f_attr:
            f_attr.write(value + "\n")

    def _read_channel_attr(self, attr):
        with open(
            os.path.join(self._channel_path, attr), "r", encoding="utf-8"
        ) as f_attr:
            return f_attr.read().strip()

    # Methods

    def enable(self):
        """Enable the PWM output."""
        self.enabled = True

    def disable(self):
        """Disable the PWM output."""
        self.enabled = False

    # Mutable properties

    def _get_period(self):
        return float(self.period_ms) / 1000

    def _set_period(self, period):
        if not isinstance(period, (int, float)):
            raise TypeError("Invalid period type, should be int.")

        self.period_ms = int(period * 1000)

    period = property(_get_period, _set_period)
    """Get or set the PWM's output period in seconds.

    Raises:
        PWMError: if an I/O or OS error occurs.
        TypeError: if value type is not int.

    :type: int, float
    """

    def _get_period_ms(self):
        return self.period_us / 1000

    def _set_period_ms(self, period_ms):
        if not isinstance(period_ms, (int, float)):
            raise TypeError("Invalid period type, should be int or float.")
        self.period_us = int(period_ms * 1000)

    period_ms = property(_get_period_ms, _set_period_ms)
    """Get or set the PWM's output period in milliseconds.

    Raises:
        PWMError: if an I/O or OS error occurs.
        TypeError: if value type is not int.

    :type: int, float
    """

    def _get_period_us(self):
        return self.period_ns / 1000

    def _set_period_us(self, period_us):
        if not isinstance(period_us, int):
            raise TypeError("Invalid period type, should be int.")

        self.period_ns = int(period_us * 1000)

    period_us = property(_get_period_us, _set_period_us)
    """Get or set the PWM's output period in microseconds.

    Raises:
        PWMError: if an I/O or OS error occurs.
        TypeError: if value type is not int.

    :type: int
    """

    def _get_period_ns(self):
        period_ns = self._read_channel_attr("period")
        try:
            period_ns = int(period_ns)
        except ValueError:
            raise PWMError(
                None, 'Unknown period value: "%s".' % period_ns
            ) from ValueError

        self._period_ns = period_ns

        return period_ns

    def _set_period_ns(self, period_ns):
        if not isinstance(period_ns, int):
            raise TypeError("Invalid period type, should be int.")

        self._write_channel_attr("period", str(period_ns))

        # Update our cached period
        self._period_ns = period_ns

    period_ns = property(_get_period_ns, _set_period_ns)
    """Get or set the PWM's output period in nanoseconds.

    Raises:
        PWMError: if an I/O or OS error occurs.
        TypeError: if value type is not int.

    :type: int
    """

    def _get_duty_cycle_ns(self):
        duty_cycle_ns_str = self._read_channel_attr("duty_cycle")

        try:
            duty_cycle_ns = int(duty_cycle_ns_str)
        except ValueError:
            raise PWMError(
                None, 'Unknown duty cycle value: "{:s}"'.format(duty_cycle_ns_str)
            ) from ValueError

        return duty_cycle_ns

    def _set_duty_cycle_ns(self, duty_cycle_ns):
        if not isinstance(duty_cycle_ns, int):
            raise TypeError("Invalid duty cycle type, should be int.")

        self._write_channel_attr("duty_cycle", str(duty_cycle_ns))

    duty_cycle_ns = property(_get_duty_cycle_ns, _set_duty_cycle_ns)
    """Get or set the PWM's output duty cycle in nanoseconds.

    Raises:
        PWMError: if an I/O or OS error occurs.
        TypeError: if value type is not int.

    :type: int
    """

    def _get_duty_cycle(self):
        return float(self.duty_cycle_ns) / self._period_ns

    def _set_duty_cycle(self, duty_cycle):
        if not isinstance(duty_cycle, (int, float)):
            raise TypeError("Invalid duty cycle type, should be int or float.")

        if not 0.0 <= duty_cycle <= 1.0:
            raise ValueError("Invalid duty cycle value, should be between 0.0 and 1.0.")

        # Convert duty cycle from ratio to nanoseconds
        self.duty_cycle_ns = int(duty_cycle * self._period_ns)

    duty_cycle = property(_get_duty_cycle, _set_duty_cycle)
    """Get or set the PWM's output duty cycle as a ratio from 0.0 to 1.0.
    Raises:
        PWMError: if an I/O or OS error occurs.
        TypeError: if value type is not int or float.
        ValueError: if value is out of bounds of 0.0 to 1.0.
    :type: int, float
    """

    def _get_frequency(self):
        return 1.0 / self.period

    def _set_frequency(self, frequency):
        if not isinstance(frequency, (int, float)):
            raise TypeError("Invalid frequency type, should be int or float.")

        self.period = 1.0 / frequency

    frequency = property(_get_frequency, _set_frequency)
    """Get or set the PWM's output frequency in Hertz.
    Raises:
        PWMError: if an I/O or OS error occurs.
        TypeError: if value type is not int or float.
    :type: int, float
    """

    def _get_polarity(self):
        return self._read_channel_attr("polarity")

    def _set_polarity(self, polarity):
        if not isinstance(polarity, str):
            raise TypeError("Invalid polarity type, should be str.")

        if polarity.lower() not in ["normal", "inversed"]:
            raise ValueError('Invalid polarity, can be: "normal" or "inversed".')

        self._write_channel_attr("polarity", polarity.lower())

    polarity = property(_get_polarity, _set_polarity)
    """Get or set the PWM's output polarity. Can be "normal" or "inversed".
    Raises:
        PWMError: if an I/O or OS error occurs.
        TypeError: if value type is not str.
        ValueError: if value is invalid.
    :type: str
    """

    def _get_enabled(self):
        enabled = self._read_channel_attr("enable")

        if enabled == "1":
            return True
        if enabled == "0":
            return False

        raise PWMError(None, 'Unknown enabled value: "{:s}"'.format(enabled))

    def _set_enabled(self, value):
        if not isinstance(value, bool):
            raise TypeError("Invalid enabled type, should be bool.")

        self._write_channel_attr("enable", "1" if value else "0")

    enabled = property(_get_enabled, _set_enabled)
    """Get or set the PWM's output enabled state.
    Raises:
        PWMError: if an I/O or OS error occurs.
        TypeError: if value type is not bool.
    :type: bool
    """

    # String representation

    def __str__(self):
        return (
            "PWM {:d}, chip {:d} (period={:f} sec, duty_cycle={:f}%,"
            " polarity={:s}, enabled={:s})".format(
                self._channel,
                self._chip,
                self.period,
                self.duty_cycle * 100,
                self.polarity,
                str(self.enabled),
            )
        )



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""
Pin definitions for the Feather RP2040 with u2if firmware.

Adafruit CircuitPython 6.2.0 on 2021-04-05; Adafruit Feather RP2040 with rp2040
>>> import board
>>> board.
A0              A1              A2              A3
D0              D1              D10             D11
D12             D13             D24             D25
D4              D5              D6              D9
I2C             LED             MISO            MOSI
NEOPIXEL        RX              SCK             SCL
SDA             SPI             TX              UART
"""

from adafruit_blinka.microcontroller.rp2040_u2if import pin

D0 = pin.GP0
D1 = pin.GP1
D4 = pin.GP6
D5 = pin.GP7
D6 = pin.GP8
D9 = pin.GP9
D10 = pin.GP10
D11 = pin.GP11
D12 = pin.GP12
D13 = pin.GP13
D24 = pin.GP24
D25 = pin.GP25

A0 = pin.GP26
A1 = pin.GP27
A2 = pin.GP28
# A3 = pin.GP29 # not currently supported in firmware

NEOPIXEL = pin.GP16

SCL = pin.GP3
SDA = pin.GP2

SCLK = SCK = pin.GP18
MOSI = pin.GP19
MISO = pin.GP20

# access u2if via pin instance to open for specifc VID/PID
# pylint:disable = protected-access
pin.GP0._u2if_open_hid(0x239A, 0x00F1)



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""I2C Class for RP2040"""
from machine import I2C as _I2C
from machine import Pin
from microcontroller.pin import i2cPorts


class I2C:
    """Custom I2C Class for RP2040"""

    def __init__(self, scl, sda, *, frequency=100000):
        for portId, portScl, portSda in i2cPorts:
            try:
                if scl == portScl and sda == portSda:
                    self._i2c = _I2C(
                        portId, sda=Pin(sda.id), scl=Pin(scl.id), freq=frequency
                    )
                    break
            except RuntimeError:
                pass
        else:
            raise ValueError(
                "No Hardware I2C on (scl,sda)={}\nValid I2C ports: {}".format(
                    (scl, sda), i2cPorts
                )
            )

    def scan(self):
        """Perform an I2C Device Scan"""
        return self._i2c.scan()

    def writeto(self, address, buffer, *, stop=True):
        "Write data to the address from the buffer"
        return self._i2c.writeto(address, buffer, stop)

    def readfrom_into(self, address, buffer, *, stop=True):
        """Read data from an address and into the buffer"""
        return self._i2c.readfrom_into(address, buffer, stop)

    def writeto_then_readfrom(
        self,
        address,
        buffer_out,
        buffer_in,
        *,
        out_start=0,
        out_end=None,
        in_start=0,
        in_end=None,
        stop=False,
    ):
        """Write data from buffer_out to an address and then
        read data from an address and into buffer_in
        """
        if out_end:
            self.writeto(address, buffer_out[out_start:out_end], stop=stop)
        else:
            self.writeto(address, buffer_out[out_start:], stop=stop)

        if not in_end:
            in_end = len(buffer_in)
        read_buffer = memoryview(buffer_in)[in_start:in_end]
        self.readfrom_into(address, read_buffer, stop=stop)



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Pin definitions for the Raspberry Pi Pico running u2if firmware"""
from adafruit_blinka.microcontroller.rp2040_u2if import pin

GP0 = pin.GP0
GP1 = pin.GP1
GP2 = pin.GP2
GP3 = pin.GP3
GP4 = pin.GP4
GP5 = pin.GP5
GP6 = pin.GP6
GP7 = pin.GP7
GP8 = pin.GP8
GP9 = pin.GP9
GP10 = pin.GP10
GP11 = pin.GP11
GP12 = pin.GP12
GP13 = pin.GP13
GP14 = pin.GP14
GP15 = pin.GP15
GP16 = pin.GP16
GP17 = pin.GP17
GP18 = pin.GP18
GP19 = pin.GP19
GP20 = pin.GP20
GP21 = pin.GP21
GP22 = pin.GP22
GP26 = pin.GP26
GP27 = pin.GP27
GP28 = pin.GP28

ADC0 = GP26
ADC1 = GP27

SCL = SCL0 = GP5
SDA = SDA0 = GP4

SCL1 = GP15
SDA1 = GP14

SCLK = SCK = SCLK0 = SCK0 = GP18
MOSI = MOSI0 = GP19
MISO = MISO0 = GP12

SCLK1 = SCK1 = GP10
MOSI1 = GP11
MISO1 = GP12

# access u2if via pin instance to open for specifc VID/PID
# pylint:disable = protected-access
pin.GP0._u2if_open_hid(0xCAFE, 0x4005)



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""NeoPixel write for Pico u2if."""

from .rp2040_u2if import rp2040_u2if


def neopixel_write(gpio, buf):
    """NeoPixel Writing Function"""

    # pad output buffer from 3 bpp to 4 bpp
    buffer = []
    for i in range(0, len(buf), 3):
        buffer.append(0)
        buffer.append(buf[i + 2])
        buffer.append(buf[i + 1])
        buffer.append(buf[i])

    rp2040_u2if.neopixel_write(gpio, buffer)



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Pin definitions for the Octavo OSD32MP1 BRK board."""

# See: https://octavosystems.com/octavosystems.com/wp-content/uploads/2020/05/Default-Pin-Mapping.pdf  # pylint: disable=line-too-long

from adafruit_blinka.microcontroller.stm32.stm32mp157 import pin

# Board pin name = OSD32MP1 pin name = STM32MP1 pin name

# fmt: off
# --------------------------------
# Header Location A
# --------------------------------
# VIN - VIN                    A01
ADC_IN0 = ANA0 = pin.PAN0    # A02
TIM8_CH2 = PI6 = pin.PI6     # A03
GPIO_PI5 = PI5 = pin.PI5     # A04
SPI2_NSS = PI0 = pin.PI0     # A05
SPI2_SCK = PI1 = pin.PI1     # A06
FDCAN1_RX = PH14 = pin.PH14  # A07
FDCAN1_TX = PH13 = pin.PH13  # A08
I2C1_SCL = PH11 = pin.PH11   # A09
TIM12_CH2 = PH9 = pin.PH9    # A10
SPI4_MOSI = PE14 = pin.PE14  # A11
SPI4_NSS = PE11 = pin.PE11   # A12
GPIO_PH8 = PH8 = pin.PH8     # A13
GPIO_PD10 = PD10 = pin.PD10  # A14
GPIO_PH5 = PH5 = pin.PH5     # A15
USART2_RX = PF4 = pin.PF4    # A16
USART2_TX = PF5 = pin.PF5    # A17
I2C2_SCL = PF1 = pin.PF1     # A18
I2C2_SDA = PG15 = pin.PG15   # A19
GPIO_PD4 = PD4 = pin.PD4     # A20
GPIO_PD6 = PD6 = pin.PD6     # A21
I2C5_SDA = PD0 = pin.PD0     # A22
TIM4_CH2 = PB7 = pin.PB7     # A23
GPIO_PE3 = PE3 = pin.PE3     # A24
GPIO_PB15 = PB15 = pin.PB15  # A25
UART7_RX = PB3 = pin.PB3     # A26
GPIO_PE5 = PE5 = pin.PE5     # A27
GPIO_PB14 = PB14 = pin.PE14  # A28
# GND - VSS                    A29
# GND - VSS                    A30

# --------------------------------
# Header Location B
# --------------------------------
# VIN - VIN                    B01
ADC_IN1 = ANA1 = pin.PAN1    # B02
GPIO_PI7 = PI7 = pin.PI7     # B03
GPIO_PI4 = PI4 = pin.PI4     # B04
SPI2_MOSI = PI3 = pin.PI3    # B05
SPI2_MISO = PI2 = pin.PI2    # B06
GPIO_PH15 = PH15 = pin.PH15  # B07
I2C1_SDA = PH12 = pin.PH12   # B08
GPIO_PH10 = PH10 = pin.PH10  # B09
GPIO_PE15 = PE15 = pin.PE15  # B10
SPI4_SCK = PE12 = pin.PE12   # B11
GPIO_PH4 = PH4 = pin.PH4     # B12
SPI4_MISO = PE13 = pin.PE13  # B13
UART8_RX = PE0 = pin.PE0     # B14
UART8_TX = PE1 = pin.PE1     # B15
GPIO_PF0 = PF0 = pin.PF0     # B16
GPIO_PE6 = PE6 = pin.PE6     # B17
GPIO_PD7 = PD7 = pin.PD7     # B18
GPIO_PD5 = PD5 = pin.PD5     # B19
I2C5_SCL = PD1 = pin.PD1     # B20
GPIO_PD3 = PD3 = pin.PD3     # B21
TIM1_CH2 = PA9 = pin.PA9     # B22
GPIO_PB9 = PB9 = pin.PB9     # B23
GPIO_PB8 = PB8 = pin.PB8     # B24
GPIO_PB4 = PB4 = pin.PB4     # B25
GPIO_PG6 = PG6 = pin.PG6     # B26
UART7_TX = PA15 = pin.PA15   # B27
GPIO_PC7 = PC7 = pin.PC7     # B28
# USB1_DP - USB_DP1            B29
# USB1_DN - USB_DM1            B30

# --------------------------------
# Header Location C
# --------------------------------
# 3.3V - PMIC_V4OUT            C01
GPIO_PD9 = PD9 = pin.PD9     # C02
GPIO_PD15 = PD15 = pin.PD15  # C03
GPIO_PD14 = PD14 = pin.PD14  # C04
GPIO_PF3 = PF3 = pin.PF3     # C05
GPIO_PC3 = PC3 = pin.PC3     # C06
GPIO_PG13 = PG13 = pin.PG13  # C07
GPIO_PG1 = PG1 = pin.PG1     # C08
GPIO_PG4 = PG4 = pin.PG4     # C09
GPIO_PF14 = PF14 = pin.PF14  # C10
GPIO_PB1 = PB1 = pin.PB1     # C11
SPI6_MISO = PZ1 = pin.PZ1    # C12
SPI6_NSS = PZ3 = pin.PZ3     # C13
# PONKEY - PWC_PONKEY          C14
# VLD02 - PMIC_LD02            C15
# VBAT - VBAT                  C16
# VSW - PMC_SWOUT              C17
# VBST - PMIC_BSTOUT           C18
GPIO_PC5 = PC5 = pin.PC5     # C19
GPIO_PA1 = PA1 = pin.PA1     # C20
GPIO_PC4 = PC4 = pin.PC4     # C21
GPIO_PB10 = PB10 = pin.PB10  # C22
UART5_RX = PB12 = pin.PB12   # C23
GPIO_PC1 = PC1 = pin.PC1     # C24
GPIO_PA4 = PA4 = pin.PA4     # C25
GPIO_PA6 = PA6 = pin.PA6     # C26
GPIO_PD11 = PD11 = pin.PD11  # C27
GPIO_PF6 = PF6 = pin.PF6     # C28
GPIO_PD12 = PD12 = pin.PD12  # C29
GPIO_PB6 = PB6 = pin.PB6     # C30

# --------------------------------
# Header Location D
# --------------------------------
# 3.V - PMIC_V4OUT             D01
# RESETN - NRST                D02
GPIO_PD8 = PD8 = pin.PD8     # D03
GPIO_PA14 = PA14 = pin.PA14  # D04
GPIO_PG12 = PG12 = pin.PG12  # D05
GPIO_PA3 = PA3 = pin.PA3     # D06
GPIO_PE2 = PE2 = pin.PE2     # D07
GPIO_PG14 = PG14 = pin.PG14  # D08
GPIO_PG2 = PG2 = pin.PG2     # D09
GPIO_PB11 = PB11 = pin.PB11  # D10
GPIO_PF13 = PF13 = pin.PF13  # D11
SPI6_SCK = PZ0 = pin.PZ0     # D12
SPI6_MOSI = PZ2 = pin.PZ2    # D13
GPIO_PC2 = PC2 = pin.PC2     # D14
GPIO_PG0 = PG0 = pin.PG0     # D15
GPIO_PG3 = PG3 = pin.PG3     # D16
GPIO_PF15 = PF15 = pin.PF15  # D17
GPIO_PF12 = PF12 = pin.PF12  # D18
GPIO_PG5 = PG5 = pin.PG5     # D19
TIM3_CH2 = PB5 = pin.PB5     # D20
GPIO_PB0 = PB0 = pin.PB0     # D21
GPIO_PA7 = PA7 = pin.PA7     # D22
UART5_TX = PB13 = pin.PB13   # D23
GPIO_PA2 = PA2 = pin.PA2     # D24
GPIO_PA5 = PA5 = pin.PA5     # D25
GPIO_PC0 = PC0 = pin.PC0     # D26
GPIO_PF11 = PF11 = pin.PF11  # D27
GPIO_PD13 = PD13 = pin.PD13  # D28
# GND - VSS                    D29
# GND - VSS                    D30
# fmt: on

# I2C defaults
SDA1 = SDA = I2C1_SDA
SCL1 = SCL = I2C1_SCL
SDA2 = I2C2_SDA
SCL2 = I2C2_SCL
SDA5 = I2C5_SDA
SCL5 = I2C5_SCL

# SPI defaults
SCK = SPI2_SCK
MOSI = SPI2_MOSI
MISO = SPI2_MISO
CS = SPI2_NSS

# Board LED's
LED1_RED = pin.PZ6
LED1_GRN = pin.PZ7
LED2_RED = pin.PI8
LED2_GRN = pin.PI9

# Console UART
UART4_TX = pin.PG11
UART4_RX = pin.PB2



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Allwinner A64 pin names"""
from adafruit_blinka.microcontroller.generic_linux.libgpiod_pin import Pin

PB0 = Pin((1, 32))
UART2_TX = PB0
PB1 = Pin((1, 33))
UART2_RX = PB1
PB2 = Pin((1, 34))
PB3 = Pin((1, 35))
PB4 = Pin((1, 36))
PB5 = Pin((1, 37))
PB6 = Pin((1, 38))
PB7 = Pin((1, 39))

PC0 = Pin((1, 64))
SPI0_MOSI = PC0
PC1 = Pin((1, 65))
SPI0_MISO = PC1
PC2 = Pin((1, 66))
SPI0_SCLK = PC2
PC3 = Pin((1, 67))
SPI0_CS = PC3
PC4 = Pin((1, 68))
PC5 = Pin((1, 69))
PC6 = Pin((1, 70))
PC7 = Pin((1, 71))
PC8 = Pin((1, 72))
PC9 = Pin((1, 73))
PC10 = Pin((1, 74))
PC11 = Pin((1, 75))
PC12 = Pin((1, 76))
PC13 = Pin((1, 77))
PC14 = Pin((1, 78))
PC15 = Pin((1, 79))
PC16 = Pin((1, 80))

PD0 = Pin((1, 96))
UART3_TX = PD0
SPI1_CS = PD0
PD1 = Pin((1, 97))
SPI1_SCLK = PD1
UART3_RX = PD1
PD2 = Pin((1, 98))
UART4_TX = PD2
SPI1_MOSI = PD2
PD3 = Pin((1, 99))
UART4_RX = PD3
SPI1_MISO = PD3
PD4 = Pin((1, 100))
PD5 = Pin((1, 101))
PD6 = Pin((1, 102))

PE14 = Pin((1, 142))
TWI2_SCL = PE14
PE15 = Pin((1, 143))
TWI2_SDA = PE15

PH2 = Pin((1, 226))
TWI1_SCL = PH2
PH3 = Pin((1, 227))
TWI1_SDA = PH3
PH4 = Pin((1, 228))
PH5 = Pin((1, 229))
PH6 = Pin((1, 230))
PH7 = Pin((1, 231))
PH8 = Pin((1, 232))
PH9 = Pin((1, 233))

PL2 = Pin((0, 2))
PL3 = Pin((0, 3))
PL8 = Pin((0, 8))
PL9 = Pin((0, 9))
PL10 = Pin((0, 10))

# ordered as i2cId, sclId, sdaId
i2cPorts = ((1, TWI1_SCL, TWI1_SDA), (2, TWI2_SCL, TWI2_SDA))

# ordered as spiId, sckId, mosiId, misoId
spiPorts = (
    (0, SPI0_SCLK, SPI0_MOSI, SPI0_MISO),
    (1, SPI1_SCLK, SPI1_MOSI, SPI1_MISO),
)
# ordered as uartId, txId, rxId
uartPorts = (
    (2, UART2_TX, UART2_RX),
    (3, UART3_TX, UART3_RX),
    (4, UART4_TX, UART4_RX),
)



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""UART Class for RP2040"""
from machine import UART as _UART
from machine import Pin
from microcontroller.pin import uartPorts

# pylint: disable=protected-access, no-self-use
class UART:
    """Custom UART Class for RP2040"""

    # pylint: disable=too-many-arguments
    def __init__(self, tx, rx, baudrate=9600, bits=8, parity=None, stop=1):
        # check tx and rx have hardware support
        for portId, txPin, rxPin in uartPorts:
            if txPin == tx and rxPin == rx:
                self._uart = _UART(
                    portId,
                    baudrate,
                    bits=bits,
                    parity=parity,
                    stop=stop,
                    tx=Pin(txPin.id),
                    rx=Pin(rxPin.id),
                )
                break
        else:
            raise ValueError(
                "No Hardware UART on (tx,rx)={}\nValid UART ports: {}".format(
                    (tx.id, rx.id), uartPorts
                )
            )

    # pylint: enable=too-many-arguments

    def read(self, nbytes=None):
        """Read from the UART"""
        return self._uart.read(nbytes)

    def readinto(self, buf, nbytes=None):
        """Read from the UART into a buffer"""
        return self._uart.readinto(buf, nbytes)

    def readline(self):
        """Read a line of characters up to a newline charater from the UART"""
        return self._uart.readline()

    def write(self, buf):
        """Write to the UART from a buffer"""
        return self._uart.write(buf)



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Pin definitions for the Orange Pi 3."""

from adafruit_blinka.microcontroller.allwinner.h6 import pin

PD26 = pin.PD26
SDA = pin.PD26
PD25 = pin.PD25
SCL = pin.PD25
PD22 = pin.PD22
PL2 = pin.PL2
PL3 = pin.PL3
PD24 = pin.PD24
UART3_RX = pin.PD24
PD18 = pin.PD18
PD23 = pin.PD23
UART3_TX = pin.PD23
PL10 = pin.PL10
PD15 = pin.PD15
PD16 = pin.PD16
PH5 = pin.PH5
PH6 = pin.PH6
PD21 = pin.PD21
PH4 = pin.PH4
SPI1_CLK = pin.PH4
PH3 = pin.PH3
SPI1_CS = pin.PH3
PL8 = pin.PL8

SCLK = pin.PH4
MOSI = pin.PH5
MISO = pin.PH6



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Pin definitions for the Rock Pi 4."""

from adafruit_blinka.microcontroller.rockchip.rk3399 import pin

D3 = pin.GPIO2_A7  # /I2C7_SDA/PIN 71/
D5 = pin.GPIO2_B0  # /I2C7_SCL/PIN 72/
D7 = pin.GPIO2_B3  # /SPI2_CLK/PIN 75/
D8 = pin.GPIO4_C4  # /UART2_TXD/PIN 148/
D10 = pin.GPIO4_C3  # /UART2_RXD/PIN 147/
D11 = pin.GPIO4_C2  # /PWM0/PIN 146/
D13 = pin.GPIO4_C6  # /PWM1/PIN 150/
D15 = pin.GPIO4_C5  # /SPDIF_TX/PIN 149/
D16 = pin.GPIO4_D2  # /PIN 154/
D17 = pin.GPIO4_D4  # /PIN 156/
D19 = pin.GPIO1_B0  # /UART4_TXD/SPI1_TXD/PIN 40/
D21 = pin.GPIO1_A7  # /UART4_RXD/SPI1_RXD/PIN 39/
D22 = pin.GPIO4_D5  # /PIN 157/
D23 = pin.GPIO1_B1  # /SPI1_CLK/PIN 41/
D24 = pin.GPIO1_B2  # /SPI1_CS/PIN 42/
D27 = pin.GPIO2_A0  # /I2C2_SDA/PIN 64/
D28 = pin.GPIO2_A1  # /I2C2_SCL/PIN 65/
D29 = pin.GPIO2_B2  # /I2C6_SCL/SPI2_TXD/PIN 74/
D31 = pin.GPIO2_B1  # /I2C6_SDA/SPI2_RXD/PIN 73/
D32 = pin.GPIO3_C0  # /SPDIF_TX/UART3_CTS/PIN 112/
D33 = pin.GPIO2_B4  # /SPI2_CS/PIN 76/
D35 = pin.GPIO4_A5  # /I2S1_LRCK_TX/PIN 133/
D36 = pin.GPIO4_A4  # /I2S1_LRCK_RX/PIN 132/
D37 = pin.GPIO4_D6  # /PIN 158/
D38 = pin.GPIO4_A6  # /I2S1_SDI/PIN 134/
D40 = pin.GPIO4_A7  # /I2S1_SDO/PIN 135/

SDA2 = D27
SCL2 = D28

SDA6 = D31
SCL6 = D29

SDA7 = D3
SCL7 = D5

SDA = SDA2
SCL = SCL2

SCLK = D7
MOSI = D29
MISO = D31
CS = D33
SCK = SCLK

UART2_TX = D8
UART2_RX = D10

UART4_TX = D19
UART4_RX = D21

UART_TX = UART2_TX
UART_RX = UART2_RX

PWM0 = pin.PWM0
PWM1 = pin.PWM1

ADC_IN0 = pin.ADC_IN0



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Allwinner H6 Pin Names"""
from adafruit_blinka.microcontroller.generic_linux.libgpiod_pin import Pin

PC16 = Pin((1, 79))

PD14 = Pin((1, 110))
PD15 = Pin((1, 111))
PD16 = Pin((1, 112))
PD17 = Pin((1, 113))
PD18 = Pin((1, 114))
PD19 = Pin((1, 115))
UART2_TX = PD19
PD20 = Pin((1, 116))
UART2_RX = PD20
PD21 = Pin((1, 117))
PD22 = Pin((1, 118))
PD23 = Pin((1, 119))
PD24 = Pin((1, 120))
PD25 = Pin((1, 121))
TWI0_SCL = PD25
PD26 = Pin((1, 122))
TWI0_SDA = PD26

PG10 = Pin((1, 202))
PG11 = Pin((1, 203))
PG12 = Pin((1, 204))
PG13 = Pin((1, 205))
PG14 = Pin((1, 206))

PH2 = Pin((1, 226))
PH3 = Pin((1, 227))
SPI1_CS = PH3
PH4 = Pin((1, 228))
SPI1_SCLK = PH4
PH5 = Pin((1, 229))
SPI1_MOSI = PH5
PH6 = Pin((1, 230))
SPI1_MISO = PH6
PH8 = Pin((1, 230))
PH9 = Pin((1, 231))

PL2 = Pin((0, 2))
PL3 = Pin((0, 3))
PL8 = Pin((0, 8))
PL9 = Pin((0, 9))
PL10 = Pin((0, 10))

i2cPorts = ((0, TWI0_SCL, TWI0_SDA),)
spiPorts = ((1, SPI1_SCLK, SPI1_MOSI, SPI1_MISO),)
uartPorts = ((2, UART2_TX, UART2_RX),)



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Helper class for use with RP2040 running u2if firmware"""
# https://github.com/execuc/u2if

import os
import time
import hid

# Use to set delay between reset and device reopen. if negative, don't reset at all
RP2040_U2IF_RESET_DELAY = float(os.environ.get("RP2040_U2IF_RESET_DELAY", 1))

# pylint: disable=import-outside-toplevel,too-many-branches,too-many-statements
# pylint: disable=too-many-arguments,too-many-function-args, too-many-public-methods


class RP2040_u2if:
    """Helper class for use with RP2040 running u2if firmware"""

    # MISC
    RESP_OK = 0x01
    SYS_RESET = 0x10

    # GPIO
    GPIO_INIT_PIN = 0x20
    GPIO_SET_VALUE = 0x21
    GPIO_GET_VALUE = 0x22

    # ADC
    ADC_INIT_PIN = 0x40
    ADC_GET_VALUE = 0x41

    # I2C
    I2C0_INIT = 0x80
    I2C0_DEINIT = 0x81
    I2C0_WRITE = 0x82
    I2C0_READ = 0x83
    I2C0_WRITE_FROM_UART = 0x84
    I2C1_INIT = I2C0_INIT + 0x10
    I2C1_DEINIT = I2C0_DEINIT + 0x10
    I2C1_WRITE = I2C0_WRITE + 0x10
    I2C1_READ = I2C0_READ + 0x10
    I2C1_WRITE_FROM_UART = I2C0_WRITE_FROM_UART + 0x10

    # SPI
    SPI0_INIT = 0x60
    SPI0_DEINIT = 0x61
    SPI0_WRITE = 0x62
    SPI0_READ = 0x63
    SPI0_WRITE_FROM_UART = 0x64
    SPI1_INIT = SPI0_INIT + 0x10
    SPI1_DEINIT = SPI0_DEINIT + 0x10
    SPI1_WRITE = SPI0_WRITE + 0x10
    SPI1_READ = SPI0_READ + 0x10
    SPI1_WRITE_FROM_UART = SPI0_WRITE_FROM_UART + 0x10

    # WS2812B (LED)
    WS2812B_INIT = 0xA0
    WS2812B_DEINIT = 0xA1
    WS2812B_WRITE = 0xA2

    # PWM
    PWM_INIT_PIN = 0x30
    PWM_DEINIT_PIN = 0x31
    PWM_SET_FREQ = 0x32
    PWM_GET_FREQ = 0x33
    PWM_SET_DUTY_U16 = 0x34
    PWM_GET_DUTY_U16 = 0x35
    PWM_SET_DUTY_NS = 0x36
    PWM_GET_DUTY_NS = 0x37

    def __init__(self):
        self._vid = None
        self._pid = None
        self._hid = None
        self._opened = False
        self._i2c_index = None
        self._spi_index = None
        self._serial = None
        self._neopixel_initialized = False
        self._uart_rx_buffer = None

    def _hid_xfer(self, report, response=True):
        """Perform HID Transfer"""
        # first byte is report ID, which =0
        # remaing bytes = 64 byte report data
        # https://github.com/libusb/hidapi/blob/083223e77952e1ef57e6b77796536a3359c1b2a3/hidapi/hidapi.h#L185
        self._hid.write(b"\0" + report + b"\0" * (64 - len(report)))
        if response:
            # return is 64 byte response report
            return self._hid.read(64)
        return None

    def _reset(self):
        self._hid_xfer(bytes([self.SYS_RESET]), False)
        time.sleep(RP2040_U2IF_RESET_DELAY)
        start = time.monotonic()
        while time.monotonic() - start < 5:
            try:
                self._hid.open(self._vid, self._pid)
            except OSError:
                time.sleep(0.1)
                continue
            return
        raise OSError("RP2040 u2if open error.")

    # ----------------------------------------------------------------
    # MISC
    # ----------------------------------------------------------------
    def open(self, vid, pid):
        """Open HID interface for given USB VID and PID."""

        if self._opened:
            return
        self._vid = vid
        self._pid = pid
        self._hid = hid.device()
        self._hid.open(self._vid, self._pid)
        if RP2040_U2IF_RESET_DELAY >= 0:
            self._reset()
        self._opened = True

    # ----------------------------------------------------------------
    # GPIO
    # ----------------------------------------------------------------
    def gpio_init_pin(self, pin_id, direction, pull):
        """Configure GPIO Pin."""
        self._hid_xfer(
            bytes(
                [
                    self.GPIO_INIT_PIN,
                    pin_id,
                    direction,
                    pull,
                ]
            )
        )

    def gpio_set_pin(self, pin_id, value):
        """Set Current GPIO Pin Value"""
        self._hid_xfer(
            bytes(
                [
                    self.GPIO_SET_VALUE,
                    pin_id,
                    int(value),
                ]
            )
        )

    def gpio_get_pin(self, pin_id):
        """Get Current GPIO Pin Value"""
        resp = self._hid_xfer(
            bytes(
                [
                    self.GPIO_GET_VALUE,
                    pin_id,
                ]
            ),
            True,
        )
        return resp[3] != 0x00

    # ----------------------------------------------------------------
    # ADC
    # ----------------------------------------------------------------
    def adc_init_pin(self, pin_id):
        """Configure ADC Pin."""
        self._hid_xfer(
            bytes(
                [
                    self.ADC_INIT_PIN,
                    pin_id,
                ]
            )
        )

    def adc_get_value(self, pin_id):
        """Get ADC value for pin."""
        resp = self._hid_xfer(
            bytes(
                [
                    self.ADC_GET_VALUE,
                    pin_id,
                ]
            ),
            True,
        )
        return int.from_bytes(resp[3 : 3 + 2], byteorder="little")

    # ----------------------------------------------------------------
    # I2C
    # ----------------------------------------------------------------
    def i2c_configure(self, baudrate, pullup=False):
        """Configure I2C."""
        if self._i2c_index is None:
            raise RuntimeError("I2C bus not initialized.")

        resp = self._hid_xfer(
            bytes(
                [
                    self.I2C0_INIT if self._i2c_index == 0 else self.I2C1_INIT,
                    0x00 if not pullup else 0x01,
                ]
            )
            + baudrate.to_bytes(4, byteorder="little"),
            True,
        )
        if resp[1] != self.RESP_OK:
            raise RuntimeError("I2C init error.")

    def i2c_set_port(self, index):
        """Set I2C port."""
        if index not in (0, 1):
            raise ValueError("I2C index must be 0 or 1.")
        self._i2c_index = index

    def _i2c_write(self, address, buffer, start=0, end=None, stop=True):
        """Write data from the buffer to an address"""
        if self._i2c_index is None:
            raise RuntimeError("I2C bus not initialized.")

        end = end if end else len(buffer)

        write_cmd = self.I2C0_WRITE if self._i2c_index == 0 else self.I2C1_WRITE
        stop_flag = 0x01 if stop else 0x00

        while (end - start) > 0:
            remain_bytes = end - start
            chunk = min(remain_bytes, 64 - 7)
            resp = self._hid_xfer(
                bytes([write_cmd, address, stop_flag])
                + remain_bytes.to_bytes(4, byteorder="little")
                + buffer[start : (start + chunk)],
                True,
            )
            if resp[1] != self.RESP_OK:
                raise RuntimeError("I2C write error")
            start += chunk

    def _i2c_read(self, address, buffer, start=0, end=None):
        """Read data from an address and into the buffer"""
        # TODO: support chunkified reads
        if self._i2c_index is None:
            raise RuntimeError("I2C bus not initialized.")

        end = end if end else len(buffer)

        read_cmd = self.I2C0_READ if self._i2c_index == 0 else self.I2C1_READ
        stop_flag = 0x01  # always stop
        read_size = end - start

        resp = self._hid_xfer(bytes([read_cmd, address, stop_flag, read_size]), True)
        if resp[1] != self.RESP_OK:
            raise RuntimeError("I2C write error")
        # move into buffer
        for i in range(read_size):
            buffer[start + i] = resp[i + 2]

    def i2c_writeto(self, address, buffer, *, start=0, end=None):
        """Write data from the buffer to an address"""
        self._i2c_write(address, buffer, start, end)

    def i2c_readfrom_into(self, address, buffer, *, start=0, end=None):
        """Read data from an address and into the buffer"""
        self._i2c_read(address, buffer, start, end)

    def i2c_writeto_then_readfrom(
        self,
        address,
        out_buffer,
        in_buffer,
        *,
        out_start=0,
        out_end=None,
        in_start=0,
        in_end=None,
    ):
        """Write data from buffer_out to an address and then
        read data from an address and into buffer_in
        """
        self._i2c_write(address, out_buffer, out_start, out_end, False)
        self._i2c_read(address, in_buffer, in_start, in_end)

    def i2c_scan(self, *, start=0, end=0x79):
        """Perform an I2C Device Scan"""
        if self._i2c_index is None:
            raise RuntimeError("I2C bus not initialized.")
        found = []
        for addr in range(start, end + 1):
            # try a write
            try:
                self.i2c_writeto(addr, b"\x00\x00\x00")
            except RuntimeError:  # no reply!
                continue
            # store if success
            found.append(addr)
        return found

    # ----------------------------------------------------------------
    # SPI
    # ----------------------------------------------------------------
    def spi_configure(self, baudrate):
        """Configure SPI."""
        if self._spi_index is None:
            raise RuntimeError("SPI bus not initialized.")

        resp = self._hid_xfer(
            bytes(
                [
                    self.SPI0_INIT if self._spi_index == 0 else self.SPI1_INIT,
                    0x00,  # mode, not yet implemented
                ]
            )
            + baudrate.to_bytes(4, byteorder="little"),
            True,
        )
        if resp[1] != self.RESP_OK:
            raise RuntimeError("SPI init error.")

    def spi_set_port(self, index):
        """Set SPI port."""
        if index not in (0, 1):
            raise ValueError("SPI index must be 0 or 1.")
        self._spi_index = index

    def spi_write(self, buffer, *, start=0, end=None):
        """SPI write."""
        if self._spi_index is None:
            raise RuntimeError("SPI bus not initialized.")

        end = end if end else len(buffer)

        write_cmd = self.SPI0_WRITE if self._spi_index == 0 else self.SPI1_WRITE

        while (end - start) > 0:
            remain_bytes = end - start
            chunk = min(remain_bytes, 64 - 3)
            resp = self._hid_xfer(
                bytes([write_cmd, chunk]) + buffer[start : (start + chunk)], True
            )
            if resp[1] != self.RESP_OK:
                raise RuntimeError("SPI write error")
            start += chunk

    def spi_readinto(self, buffer, *, start=0, end=None, write_value=0):
        """SPI readinto."""
        if self._spi_index is None:
            raise RuntimeError("SPI bus not initialized.")

        end = end if end else len(buffer)
        read_cmd = self.SPI0_READ if self._spi_index == 0 else self.SPI1_READ
        read_size = end - start

        resp = self._hid_xfer(bytes([read_cmd, write_value, read_size]), True)
        if resp[1] != self.RESP_OK:
            raise RuntimeError("SPI write error")
        # move into buffer
        for i in range(read_size):
            buffer[start + i] = resp[i + 2]

    def spi_write_readinto(
        self,
        buffer_out,
        buffer_in,
        *,
        out_start=0,
        out_end=None,
        in_start=0,
        in_end=None,
    ):
        """SPI write and readinto."""
        raise NotImplementedError("SPI write_readinto Not implemented")

    # ----------------------------------------------------------------
    # NEOPIXEL
    # ----------------------------------------------------------------
    def neopixel_write(self, gpio, buf):
        """NeoPixel write."""
        # open serial (data is sent over this)
        if self._serial is None:
            import serial
            import serial.tools.list_ports

            ports = serial.tools.list_ports.comports()
            for port in ports:
                if port.vid == self._vid and port.pid == self._pid:
                    self._serial = serial.Serial(port.device)
                    break
        if self._serial is None:
            raise RuntimeError("Could not find Pico com port.")

        # init
        if not self._neopixel_initialized:
            # deinit any current setup
            # pylint: disable=protected-access
            self._hid_xfer(bytes([self.WS2812B_DEINIT]))
            resp = self._hid_xfer(
                bytes(
                    [
                        self.WS2812B_INIT,
                        gpio._pin.id,
                    ]
                ),
                True,
            )
            if resp[1] != self.RESP_OK:
                raise RuntimeError("Neopixel init error")
            self._neopixel_initialized = True

        self._serial.reset_output_buffer()

        # write
        # command is done over HID
        remain_bytes = len(buf)
        resp = self._hid_xfer(
            bytes([self.WS2812B_WRITE]) + remain_bytes.to_bytes(4, byteorder="little"),
            True,
        )
        if resp[1] != self.RESP_OK:
            # pylint: disable=no-else-raise
            if resp[2] == 0x01:
                raise RuntimeError(
                    "Neopixel write error : too many pixel for the firmware."
                )
            elif resp[2] == 0x02:
                raise RuntimeError(
                    "Neopixel write error : transfer already in progress."
                )
            else:
                raise RuntimeError("Neopixel write error.")
        # buffer is sent over serial
        self._serial.write(buf)
        # hack (see u2if)
        if len(buf) % 64 == 0:
            self._serial.write([0])
        self._serial.flush()
        # polling loop to wait for write complete?
        time.sleep(0.1)
        resp = self._hid.read(64)
        while resp[0] != self.WS2812B_WRITE:
            resp = self._hid.read(64)
        if resp[1] != self.RESP_OK:
            raise RuntimeError("Neopixel write (flush) error.")

    # ----------------------------------------------------------------
    # PWM
    # ----------------------------------------------------------------
    # pylint: disable=unused-argument
    def pwm_configure(self, pin, frequency=500, duty_cycle=0, variable_frequency=False):
        """Configure PWM."""
        self.pwm_deinit(pin)
        resp = self._hid_xfer(bytes([self.PWM_INIT_PIN, pin.id]), True)
        if resp[1] != self.RESP_OK:
            raise RuntimeError("PWM init error.")

        self.pwm_set_frequency(pin, frequency)
        self.pwm_set_duty_cycle(pin, duty_cycle)

    def pwm_deinit(self, pin):
        """Deinit PWM."""
        self._hid_xfer(bytes([self.PWM_DEINIT_PIN, pin.id]))

    def pwm_get_frequency(self, pin):
        """PWM get freq."""
        resp = self._hid_xfer(bytes([self.PWM_GET_FREQ, pin.id]), True)
        if resp[1] != self.RESP_OK:
            raise RuntimeError("PWM get frequency error.")
        return int.from_bytes(resp[3 : 3 + 4], byteorder="little")

    def pwm_set_frequency(self, pin, frequency):
        """PWM set freq."""
        resp = self._hid_xfer(
            bytes([self.PWM_SET_FREQ, pin.id])
            + frequency.to_bytes(4, byteorder="little"),
            True,
        )
        if resp[1] != self.RESP_OK:
            # pylint: disable=no-else-raise
            if resp[3] == 0x01:
                raise RuntimeError("PWM different frequency on same slice.")
            elif resp[3] == 0x02:
                raise RuntimeError("PWM frequency too low.")
            elif resp[3] == 0x03:
                raise RuntimeError("PWM frequency too high.")
            else:
                raise RuntimeError("PWM frequency error.")

    def pwm_get_duty_cycle(self, pin):
        """PWM get duty cycle."""
        resp = self._hid_xfer(bytes([self.PWM_GET_DUTY_U16, pin.id]), True)
        if resp[1] != self.RESP_OK:
            raise RuntimeError("PWM get duty cycle error.")
        return int.from_bytes(resp[3 : 3 + 4], byteorder="little")

    def pwm_set_duty_cycle(self, pin, duty_cycle):
        """PWM set duty cycle."""
        resp = self._hid_xfer(
            bytes([self.PWM_SET_DUTY_U16, pin.id])
            + duty_cycle.to_bytes(2, byteorder="little"),
            True,
        )
        if resp[1] != self.RESP_OK:
            raise RuntimeError("PWM set duty cycle error.")


rp2040_u2if = RP2040_u2if()



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Broadcom BCM2711 pin names"""
from RPi import GPIO
from adafruit_blinka.microcontroller.bcm283x.pin import Pin

GPIO.setmode(GPIO.BCM)  # Use BCM pins D4 = GPIO #4
GPIO.setwarnings(False)  # shh!

D0 = Pin(0)
D1 = Pin(1)

D2 = Pin(2)
SDA = Pin(2)
D3 = Pin(3)
SCL = Pin(3)

D4 = Pin(4)
D5 = Pin(5)
D6 = Pin(6)

D7 = Pin(7)
CE1 = Pin(7)
D8 = Pin(8)
CE0 = Pin(8)
D9 = Pin(9)
MISO = Pin(9)
D10 = Pin(10)
MOSI = Pin(10)
D11 = Pin(11)
SCLK = Pin(11)  # Raspberry Pi naming
SCK = Pin(11)  # CircuitPython naming

D12 = Pin(12)
D13 = Pin(13)

D14 = Pin(14)
TXD = Pin(14)
D15 = Pin(15)
RXD = Pin(15)

D16 = Pin(16)
D17 = Pin(17)
D18 = Pin(18)
D19 = Pin(19)
MISO_1 = Pin(19)
D20 = Pin(20)
MOSI_1 = Pin(20)
D21 = Pin(21)
SCLK_1 = Pin(21)
SCK_1 = Pin(21)
D22 = Pin(22)
D23 = Pin(23)
D24 = Pin(24)
D25 = Pin(25)
D26 = Pin(26)
D27 = Pin(27)
D28 = Pin(28)
D29 = Pin(29)
D30 = Pin(30)
D31 = Pin(31)
D32 = Pin(32)
D33 = Pin(33)
D34 = Pin(34)
D35 = Pin(35)
D36 = Pin(36)
D37 = Pin(37)
D38 = Pin(38)
D39 = Pin(39)
D40 = Pin(40)
MISO_2 = Pin(40)
D41 = Pin(41)
MOSI_2 = Pin(41)
D42 = Pin(42)
SCLK_2 = Pin(42)
SCK_2 = Pin(43)
D43 = Pin(43)
D44 = Pin(44)
D45 = Pin(45)

# ordered as spiId, sckId, mosiId, misoId
spiPorts = (
    (0, SCLK, MOSI, MISO),
    (6, SCLK_1, MOSI_1, MISO_1),
    (2, SCLK_2, MOSI_2, MISO_2),
    (3, D3, D2, D1),
    (4, D7, D6, D5),
    (5, D15, D14, D13),
)

# ordered as uartId, txId, rxId
uartPorts = ((1, TXD, RXD),)

# These are the known hardware I2C ports / pins.
# For software I2C ports created with the i2c-gpio overlay, see:
#     https://github.com/adafruit/Adafruit_Python_Extended_Bus
i2cPorts = (
    (1, SCL, SDA),
    (0, D1, D0),  # both pi 1 and pi 2 i2c ports!
    (10, D45, D44),  # internal i2c bus for the CM4
)



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""AmLogic s922x pin names"""
# pylint: disable=wildcard-import,unused-wildcard-import
from adafruit_blinka.microcontroller.amlogic.meson_g12_common.pin import *



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Generic RP2040 pin names"""
from .rp2040_u2if import rp2040_u2if


class Pin:
    """A basic Pin class for use with RP2040 with u2if firmware."""

    # pin modes
    IN = 0
    OUT = 1
    # pin values
    LOW = 0
    HIGH = 1
    # pin pulls
    PULL_NONE = 0
    PULL_UP = 1
    PULL_DOWN = 2

    def __init__(self, pin_id=None):
        self.id = pin_id
        self._mode = None
        self._pull = None

    # pylint:disable = no-self-use
    def _u2if_open_hid(self, vid, pid):
        rp2040_u2if.open(vid, pid)

    def init(self, mode=IN, pull=PULL_NONE):
        """Initialize the Pin"""
        pull = Pin.PULL_NONE if pull is None else pull
        if self.id is None:
            raise RuntimeError("Can not init a None type pin.")
        if mode not in (Pin.IN, Pin.OUT):
            raise ValueError("Incorrect mode value.")
        if pull not in (Pin.PULL_NONE, Pin.PULL_UP, Pin.PULL_DOWN):
            raise ValueError("Incorrect pull value.")

        rp2040_u2if.gpio_init_pin(self.id, mode, pull)

        self._mode = mode
        self._pull = pull

    def value(self, val=None):
        """Set or return the Pin Value"""
        # Digital In / Out
        if self._mode in (Pin.IN, Pin.OUT):
            # digital read
            if val is None:
                return rp2040_u2if.gpio_get_pin(self.id)
            # digital write
            if val in (Pin.LOW, Pin.HIGH):
                rp2040_u2if.gpio_set_pin(self.id, val)
                return None
            # nope
            raise ValueError("Invalid value for pin.")

        raise RuntimeError(
            "No action for mode {} with value {}".format(self._mode, val)
        )


# create pin instances for each pin
GP0 = Pin(0)
GP1 = Pin(1)
GP2 = Pin(2)
GP3 = Pin(3)
GP4 = Pin(4)
GP5 = Pin(5)
GP6 = Pin(6)
GP7 = Pin(7)
GP8 = Pin(8)
GP9 = Pin(9)
GP10 = Pin(10)
GP11 = Pin(11)
GP12 = Pin(12)
GP13 = Pin(13)
GP14 = Pin(14)
GP15 = Pin(15)
GP16 = Pin(16)
GP17 = Pin(17)
GP18 = Pin(18)
GP19 = Pin(19)
GP20 = Pin(20)
GP21 = Pin(21)
GP22 = Pin(22)
GP23 = Pin(23)
GP24 = Pin(24)
GP25 = Pin(25)
GP26 = Pin(26)
GP27 = Pin(27)
GP28 = Pin(28)
GP29 = Pin(29)



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""
`usb_hid` - support for usb hid devices via usb_gadget driver
===========================================================
See `CircuitPython:usb_hid` in CircuitPython for more details.
For now using report ids in the descriptor

# regarding usb_gadget see https://www.kernel.org/doc/Documentation/usb/gadget_configfs.txt
* Author(s): Björn Bösel
"""

from typing import Sequence
from pathlib import Path
import os
import atexit
import sys

for module in ["dwc2", "libcomposite"]:
    if Path("/proc/modules").read_text(encoding="utf-8").find(module) == -1:
        raise Exception(
            "%s module not present in your kernel. did you insmod it?" % module
        )
this = sys.modules[__name__]

this.gadget_root = "/sys/kernel/config/usb_gadget/adafruit-blinka"
this.boot_device = 0
this.devices = []


class Device:
    """
    HID Device specification: see
    https://github.com/adafruit/circuitpython/blob/main/shared-bindings/usb_hid/Device.c
    """

    def __init__(
        self,
        *,
        descriptor: bytes,
        usage_page: int,
        usage: int,
        report_ids: Sequence[int],
        in_report_lengths: Sequence[int],
        out_report_lengths: Sequence[int],
    ) -> None:
        self.out_report_lengths = out_report_lengths
        self.in_report_lengths = in_report_lengths
        self.report_ids = report_ids
        self.usage = usage
        self.usage_page = usage_page
        self.descriptor = descriptor
        self._last_received_report = None

    def send_report(self, report: bytearray, report_id: int = None):
        """Send an HID report. If the device descriptor specifies zero or one report id's,
        you can supply `None` (the default) as the value of ``report_id``.
        Otherwise you must specify which report id to use when sending the report.
        """
        report_id = report_id or self.report_ids[0]
        device_path = self.get_device_path(report_id)
        with open(device_path, "rb+", encoding="utf-8") as fd:
            if report_id > 0:
                report = bytearray(report_id.to_bytes(1, "big")) + report
            fd.write(report)

    @property
    def last_received_report(
        self,
    ) -> bytes:
        """The HID OUT report as a `bytes` (read-only). `None` if nothing received.
        Same as `get_last_received_report()` with no argument.

        Deprecated: will be removed in CircutPython 8.0.0. Use `get_last_received_report()` instead.
        """
        return self.get_last_received_report()

    def get_last_received_report(self, report_id=None) -> bytes:
        """Get the last received HID OUT or feature report for the given report ID.
        The report ID may be omitted if there is no report ID, or only one report ID.
        Return `None` if nothing received.
        """
        device_path = self.get_device_path(report_id or self.report_ids[0])
        with open(device_path, "rb+", encoding="utf-8") as fd:
            os.set_blocking(fd.fileno(), False)
            report = fd.read(self.out_report_lengths[0])
            if report is not None:
                self._last_received_report = report
        return self._last_received_report

    def get_device_path(self, report_id):
        """
        translates the /dev/hidg device from the report id
        """
        device = (
            Path(
                "%s/functions/hid.usb%s/dev"
                % (this.gadget_root, report_id or self.report_ids[0])
            )
            .read_text(encoding="utf-8")
            .strip()
            .split(":")[1]
        )
        device_path = "/dev/hidg%s" % device
        return device_path

    KEYBOARD = None
    MOUSE = None
    CONSUMER_CONTROL = None


Device.KEYBOARD = Device(
    descriptor=bytes(
        (
            0x05,
            0x01,  # usage page (generic desktop ctrls)
            0x09,
            0x06,  # usage (keyboard)
            0xA1,
            0x01,  # collection (application)
            0x85,
            0x01,  # Report ID (1)
            0x05,
            0x07,  # usage page (kbrd/keypad)
            0x19,
            0xE0,  # usage minimum (0xe0)
            0x29,
            0xE7,  # usage maximum (0xe7)
            0x15,
            0x00,  # logical minimum (0)
            0x25,
            0x01,  # logical maximum (1)
            0x75,
            0x01,  # report size (1)
            0x95,
            0x08,  # report count (8)
            0x81,
            0x02,  # input (data,var,abs,no wrap,linear,preferred state,no null position)
            0x95,
            0x01,  # report count (1)
            0x75,
            0x08,  # report size (8)
            0x81,
            0x01,  # input (const,array,abs,no wrap,linear,preferred state,no null position)
            0x95,
            0x03,  # report count (3)
            0x75,
            0x01,  # report size (1)
            0x05,
            0x08,  # usage page (leds)
            0x19,
            0x01,  # usage minimum (num lock)
            0x29,
            0x05,  # usage maximum (kana)
            0x91,
            0x02,  # output
            # (data,var,abs,no wrap,linear,preferred state,no null position,non-volatile)
            0x95,
            0x01,  # report count (1)
            0x75,
            0x05,  # report size (5)
            0x91,
            0x01,  # output
            # (const,array,abs,no wrap,linear,preferred state,no null position,non-volatile)
            0x95,
            0x06,  # report count (6)
            0x75,
            0x08,  # report size (8)
            0x15,
            0x00,  # logical minimum (0)
            0x26,
            0xFF,
            0x00,  # logical maximum (255)
            0x05,
            0x07,  # usage page (kbrd/keypad)
            0x19,
            0x00,  # usage minimum (0x00)
            0x2A,
            0xFF,
            0x00,  # usage maximum (0xff)
            0x81,
            0x00,  # input (data,array,abs,no wrap,linear,preferred state,no null position)
            0xC0,  # end collection
        )
    ),
    usage_page=0x1,
    usage=0x6,
    report_ids=[0x1],
    in_report_lengths=[8],
    out_report_lengths=[1],
)
Device.MOUSE = Device(
    descriptor=bytes(
        (
            0x05,
            0x01,  # Usage Page (Generic Desktop Ctrls)
            0x09,
            0x02,  # Usage (Mouse)
            0xA1,
            0x01,  # Collection (Application)
            0x85,
            0x02,  # Report ID (2)
            0x09,
            0x01,  # Usage (Pointer)
            0xA1,
            0x00,  # Collection (Physical)
            0x05,
            0x09,  # Usage Page (Button)
            0x19,
            0x01,  # Usage Minimum (0x01)
            0x29,
            0x05,  # Usage Maximum (0x05)
            0x15,
            0x00,  # Logical Minimum (0)
            0x25,
            0x01,  # Logical Maximum (1)
            0x95,
            0x05,  # Report Count (5)
            0x75,
            0x01,  # Report Size (1)
            0x81,
            0x02,  # Input (Data,Var,Abs,No Wrap,Linear,Preferred State,No Null Position)
            0x95,
            0x01,  # Report Count (1)
            0x75,
            0x03,  # Report Size (3)
            0x81,
            0x01,  # Input (Const,Array,Abs,No Wrap,Linear,Preferred State,No Null Position)
            0x05,
            0x01,  # Usage Page (Generic Desktop Ctrls)
            0x09,
            0x30,  # Usage (X)
            0x09,
            0x31,  # Usage (Y)
            0x15,
            0x81,  # Logical Minimum (-127)
            0x25,
            0x7F,  # Logical Maximum (127)
            0x75,
            0x08,  # Report Size (8)
            0x95,
            0x02,  # Report Count (2)
            0x81,
            0x06,  # Input (Data,Var,Rel,No Wrap,Linear,Preferred State,No Null Position)
            0x09,
            0x38,  # Usage (Wheel)
            0x15,
            0x81,  # Logical Minimum (-127)
            0x25,
            0x7F,  # Logical Maximum (127)
            0x75,
            0x08,  # Report Size (8)
            0x95,
            0x01,  # Report Count (1)
            0x81,
            0x06,  # Input (Data,Var,Rel,No Wrap,Linear,Preferred State,No Null Position)
            0xC0,  # End Collection
            0xC0,  # End Collection
        )
    ),
    usage_page=0x1,
    usage=0x02,
    report_ids=[0x02],
    in_report_lengths=[4],
    out_report_lengths=[0],
)

Device.CONSUMER_CONTROL = Device(
    descriptor=bytes(
        (
            0x05,
            0x0C,  # Usage Page (Consumer)
            0x09,
            0x01,  # Usage (Consumer Control)
            0xA1,
            0x01,  # Collection (Application)
            0x85,
            0x03,  # Report ID (3)
            0x75,
            0x10,  # Report Size (16)
            0x95,
            0x01,  # Report Count (1)
            0x15,
            0x01,  # Logical Minimum (1)
            0x26,
            0x8C,
            0x02,  # Logical Maximum (652)
            0x19,
            0x01,  # Usage Minimum (Consumer Control)
            0x2A,
            0x8C,
            0x02,  # Usage Maximum (AC Send)
            0x81,
            0x00,  # Input (Data,Array,Abs,No Wrap,Linear,Preferred State,No Null Position)
            0xC0,  # End Collection
        )
    ),
    usage_page=0x0C,
    usage=0x01,
    report_ids=[3],
    in_report_lengths=[2],
    out_report_lengths=[0],
)

Device.BOOT_KEYBOARD = Device(
    descriptor=bytes(
        (
            0x05,
            0x01,  # usage page (generic desktop ctrls)
            0x09,
            0x06,  # usage (keyboard)
            0xA1,
            0x01,  # collection (application)
            0x05,
            0x07,  # usage page (kbrd/keypad)
            0x19,
            0xE0,  # usage minimum (0xe0)
            0x29,
            0xE7,  # usage maximum (0xe7)
            0x15,
            0x00,  # logical minimum (0)
            0x25,
            0x01,  # logical maximum (1)
            0x75,
            0x01,  # report size (1)
            0x95,
            0x08,  # report count (8)
            0x81,
            0x02,  # input (data,var,abs,no wrap,linear,preferred state,no null position)
            0x95,
            0x01,  # report count (1)
            0x75,
            0x08,  # report size (8)
            0x81,
            0x01,  # input (const,array,abs,no wrap,linear,preferred state,no null position)
            0x95,
            0x03,  # report count (3)
            0x75,
            0x01,  # report size (1)
            0x05,
            0x08,  # usage page (leds)
            0x19,
            0x01,  # usage minimum (num lock)
            0x29,
            0x05,  # usage maximum (kana)
            0x91,
            0x02,  # output
            # (data,var,abs,no wrap,linear,preferred state,no null position,non-volatile)
            0x95,
            0x01,  # report count (1)
            0x75,
            0x05,  # report size (5)
            0x91,
            0x01,  # output
            # (const,array,abs,no wrap,linear,preferred state,no null position,non-volatile)
            0x95,
            0x06,  # report count (6)
            0x75,
            0x08,  # report size (8)
            0x15,
            0x00,  # logical minimum (0)
            0x26,
            0xFF,
            0x00,  # logical maximum (255)
            0x05,
            0x07,  # usage page (kbrd/keypad)
            0x19,
            0x00,  # usage minimum (0x00)
            0x2A,
            0xFF,
            0x00,  # usage maximum (0xff)
            0x81,
            0x00,  # input (data,array,abs,no wrap,linear,preferred state,no null position)
            0xC0,  # end collection
        )
    ),
    usage_page=0x1,
    usage=0x6,
    report_ids=[0x0],
    in_report_lengths=[8],
    out_report_lengths=[1],
)
Device.BOOT_MOUSE = Device(
    descriptor=bytes(
        (
            0x05,
            0x01,  # Usage Page (Generic Desktop Ctrls)
            0x09,
            0x02,  # Usage (Mouse)
            0xA1,
            0x01,  # Collection (Application)
            0x09,
            0x01,  # Usage (Pointer)
            0xA1,
            0x00,  # Collection (Physical)
            0x05,
            0x09,  # Usage Page (Button)
            0x19,
            0x01,  # Usage Minimum (0x01)
            0x29,
            0x05,  # Usage Maximum (0x05)
            0x15,
            0x00,  # Logical Minimum (0)
            0x25,
            0x01,  # Logical Maximum (1)
            0x95,
            0x05,  # Report Count (5)
            0x75,
            0x01,  # Report Size (1)
            0x81,
            0x02,  # Input (Data,Var,Abs,No Wrap,Linear,Preferred State,No Null Position)
            0x95,
            0x01,  # Report Count (1)
            0x75,
            0x03,  # Report Size (3)
            0x81,
            0x01,  # Input (Const,Array,Abs,No Wrap,Linear,Preferred State,No Null Position)
            0x05,
            0x01,  # Usage Page (Generic Desktop Ctrls)
            0x09,
            0x30,  # Usage (X)
            0x09,
            0x31,  # Usage (Y)
            0x15,
            0x81,  # Logical Minimum (-127)
            0x25,
            0x7F,  # Logical Maximum (127)
            0x75,
            0x08,  # Report Size (8)
            0x95,
            0x02,  # Report Count (2)
            0x81,
            0x06,  # Input (Data,Var,Rel,No Wrap,Linear,Preferred State,No Null Position)
            0x09,
            0x38,  # Usage (Wheel)
            0x15,
            0x81,  # Logical Minimum (-127)
            0x25,
            0x7F,  # Logical Maximum (127)
            0x75,
            0x08,  # Report Size (8)
            0x95,
            0x01,  # Report Count (1)
            0x81,
            0x06,  # Input (Data,Var,Rel,No Wrap,Linear,Preferred State,No Null Position)
            0xC0,  # End Collection
            0xC0,  # End Collection
        )
    ),
    usage_page=0x1,
    usage=0x02,
    report_ids=[0],
    in_report_lengths=[4],
    out_report_lengths=[0],
)


def disable() -> None:
    """Do not present any USB HID devices to the host computer.
    Can be called in ``boot.py``, before USB is connected.
    The HID composite device is normally enabled by default,
    but on some boards with limited endpoints, including STM32F4,
    it is disabled by default. You must turn off another USB device such
    as `usb_cdc` or `storage` to free up endpoints for use by `usb_hid`.
    """
    try:
        Path("%s/UDC" % this.gadget_root).write_text("", encoding="utf-8")
    except FileNotFoundError:
        pass
    for symlink in Path(this.gadget_root).glob("configs/**/hid.usb*"):
        symlink.unlink()

    for strings_file in Path(this.gadget_root).rglob("configs/*/strings/*/*"):
        if strings_file.is_dir():
            strings_file.rmdir()

    for strings_file in Path(this.gadget_root).rglob("configs/*/strings/*"):
        if strings_file.is_dir():
            strings_file.rmdir()
    for config_dir in Path(this.gadget_root).rglob("configs/*"):
        if config_dir.is_dir():
            config_dir.rmdir()
    for function_dir in Path(this.gadget_root).rglob("functions/*"):
        if function_dir.is_dir():
            function_dir.rmdir()
    try:
        Path(this.gadget_root).rmdir()
    except FileNotFoundError:
        pass
    this.devices = []


atexit.register(disable)


def enable(requested_devices: Sequence[Device], boot_device: int = 0) -> None:
    """Specify which USB HID devices that will be available.
    Can be called in ``boot.py``, before USB is connected.

    :param Sequence devices: `Device` objects.
      If `devices` is empty, HID is disabled. The order of the ``Devices``
      may matter to the host. For instance, for MacOS, put the mouse device
      before any Gamepad or Digitizer HID device or else it will not work.
    :param int boot_device: If non-zero, inform the host that support for a
      a boot HID device is available.
      If ``boot_device=1``, a boot keyboard is available.
      If ``boot_device=2``, a boot mouse is available. No other values are allowed.
      See below.

    If you enable too many devices at once, you will run out of USB endpoints.
    The number of available endpoints varies by microcontroller.
    CircuitPython will go into safe mode after running ``boot.py`` to inform you if
    not enough endpoints are available.

    **Boot Devices**

    Boot devices implement a fixed, predefined report descriptor, defined in
    https://www.usb.org/sites/default/files/hid1_12.pdf, Appendix B. A USB host
    can request to use the boot device if the USB device says it is available.
    Usually only a BIOS or other kind of limited-functionality
    host needs boot keyboard support.

    For example, to make a boot keyboard available, you can use this code::

      usb_hid.enable((Device.KEYBOARD), boot_device=1)  # 1 for a keyboard

    If the host requests the boot keyboard, the report descriptor provided by `Device.KEYBOARD`
    will be ignored, and the predefined report descriptor will be used.
    But if the host does not request the boot keyboard,
    the descriptor provided by `Device.KEYBOARD` will be used.

    The HID boot device must usually be the first or only device presented by CircuitPython.
    The HID device will be USB interface number 0.
    To make sure it is the first device, disable other USB devices, including CDC and MSC
    (CIRCUITPY).
    If you specify a non-zero ``boot_device``, and it is not the first device, CircuitPython
    will enter safe mode to report this error.
    """
    this.boot_device = boot_device

    if len(requested_devices) == 0:
        disable()
        return

    if boot_device == 1:
        requested_devices = [Device.BOOT_KEYBOARD]
    if boot_device == 2:
        requested_devices = [Device.BOOT_MOUSE]

    # """
    # 1. Creating the gadgets
    # -----------------------
    #
    # For each gadget to be created its corresponding directory must be created::
    #
    #     $ mkdir $CONFIGFS_HOME/usb_gadget/<gadget name>
    #
    # e.g.::
    #
    #     $ mkdir $CONFIGFS_HOME/usb_gadget/g1
    #
    #     ...
    #     ...
    #     ...
    #
    #     $ cd $CONFIGFS_HOME/usb_gadget/g1
    #
    # Each gadget needs to have its vendor id <VID> and product id <PID> specified::
    #
    #     $ echo <VID> > idVendor
    #     $ echo <PID> > idProduct
    #
    # A gadget also needs its serial number, manufacturer and product strings.
    # In order to have a place to store them, a strings subdirectory must be created
    # for each language, e.g.::
    #
    #     $ mkdir strings/0x409
    #
    # Then the strings can be specified::
    #
    #     $ echo <serial number> > strings/0x409/serialnumber
    #     $ echo <manufacturer> > strings/0x409/manufacturer
    #     $ echo <product> > strings/0x409/product
    # """
    Path("%s/functions" % this.gadget_root).mkdir(parents=True, exist_ok=True)
    Path("%s/configs" % this.gadget_root).mkdir(parents=True, exist_ok=True)
    Path("%s/bcdDevice" % this.gadget_root).write_text(
        "%s" % 1, encoding="utf-8"
    )  # Version 1.0.0
    Path("%s/bcdUSB" % this.gadget_root).write_text(
        "%s" % 0x0200, encoding="utf-8"
    )  # USB 2.0
    Path("%s/bDeviceClass" % this.gadget_root).write_text(
        "%s" % 0x00, encoding="utf-8"
    )  # multipurpose i guess?
    Path("%s/bDeviceProtocol" % this.gadget_root).write_text(
        "%s" % 0x00, encoding="utf-8"
    )
    Path("%s/bDeviceSubClass" % this.gadget_root).write_text(
        "%s" % 0x00, encoding="utf-8"
    )
    Path("%s/bMaxPacketSize0" % this.gadget_root).write_text(
        "%s" % 0x08, encoding="utf-8"
    )
    Path("%s/idProduct" % this.gadget_root).write_text(
        "%s" % 0x0104, encoding="utf-8"
    )  # Multifunction Composite Gadget
    Path("%s/idVendor" % this.gadget_root).write_text(
        "%s" % 0x1D6B, encoding="utf-8"
    )  # Linux Foundation
    # """
    # 2. Creating the configurations
    # ------------------------------
    #
    # Each gadget will consist of a number of configurations, their corresponding
    # directories must be created:
    #
    # $ mkdir configs/<name>.<number>
    #
    # where <name> can be any string which is legal in a filesystem and the
    # <number> is the configuration's number, e.g.::
    #
    #     $ mkdir configs/c.1
    #
    #     ...
    #     ...
    #     ...
    #
    # Each configuration also needs its strings, so a subdirectory must be created
    # for each language, e.g.::
    #
    #     $ mkdir configs/c.1/strings/0x409
    #
    # Then the configuration string can be specified::
    #
    #     $ echo <configuration> > configs/c.1/strings/0x409/configuration
    #
    # Some attributes can also be set for a configuration, e.g.::
    #
    #     $ echo 120 > configs/c.1/MaxPower
    #     """

    for device in requested_devices:
        config_root = "%s/configs/device.1" % this.gadget_root
        Path("%s/" % config_root).mkdir(parents=True, exist_ok=True)
        Path("%s/strings/0x409" % config_root).mkdir(parents=True, exist_ok=True)
        Path("%s/strings/0x409/configuration" % config_root).write_text(
            "my configuration", encoding="utf-8"
        )
        Path("%s/MaxPower" % config_root).write_text("150", encoding="utf-8")
        Path("%s/bmAttributes" % config_root).write_text("%s" % 0x080, encoding="utf-8")
        this.devices.append(device)
        # """
        # 3. Creating the functions
        # -------------------------
        #
        # The gadget will provide some functions, for each function its corresponding
        # directory must be created::
        #
        #     $ mkdir functions/<name>.<instance name>
        #
        # where <name> corresponds to one of allowed function names and instance name
        # is an arbitrary string allowed in a filesystem, e.g.::
        #
        #   $ mkdir functions/ncm.usb0 # usb_f_ncm.ko gets loaded with request_module()
        #
        #   ...
        #   ...
        #   ...
        #
        # Each function provides its specific set of attributes, with either read-only
        # or read-write access. Where applicable they need to be written to as
        # appropriate.
        # Please refer to Documentation/ABI/*/configfs-usb-gadget* for more information.  """
        for report_index, report_id in enumerate(device.report_ids):
            function_root = "%s/functions/hid.usb%s" % (this.gadget_root, report_id)
            try:
                Path("%s/" % function_root).mkdir(parents=True)
            except FileExistsError:
                continue
            Path("%s/protocol" % function_root).write_text(
                "%s" % report_id, encoding="utf-8"
            )
            Path("%s/report_length" % function_root).write_text(
                "%s" % device.in_report_lengths[report_index], encoding="utf-8"
            )
            Path("%s/subclass" % function_root).write_text("%s" % 1, encoding="utf-8")
            Path("%s/report_desc" % function_root).write_bytes(device.descriptor)
            # """
            # 4. Associating the functions with their configurations
            # ------------------------------------------------------
            #
            # At this moment a number of gadgets is created, each of which has a number of
            # configurations specified and a number of functions available. What remains
            # is specifying which function is available in which configuration (the same
            # function can be used in multiple configurations). This is achieved with
            # creating symbolic links::
            #
            #     $ ln -s functions/<name>.<instance name> configs/<name>.<number>
            #
            # e.g.::
            #
            #     $ ln -s functions/ncm.usb0 configs/c.1  """
            try:
                Path("%s/hid.usb%s" % (config_root, report_id)).symlink_to(
                    function_root
                )
            except FileNotFoundError:
                pass
    # """ 5. Enabling the gadget
    # ----------------------
    # Such a gadget must be finally enabled so that the USB host can enumerate it.
    #
    # In order to enable the gadget it must be bound to a UDC (USB Device
    # Controller)::
    #
    #     $ echo <udc name> > UDC
    #
    # where <udc name> is one of those found in /sys/class/udc/*
    # e.g.::
    #
    # $ echo s3c-hsotg > UDC  """
    udc = next(Path("/sys/class/udc/").glob("*"))
    Path("%s/UDC" % this.gadget_root).write_text("%s" % udc.name, encoding="utf-8")



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Pin definitions for the QT2040 Trinkey with u2if firmware."""

from adafruit_blinka.microcontroller.rp2040_u2if import pin

BUTTON = pin.GP12

SCL = pin.GP17
SDA = pin.GP16

NEOPIXEL = pin.GP27

# access u2if via pin instance to open for specifc VID/PID
# pylint:disable = protected-access
pin.GP0._u2if_open_hid(0x239A, 0x0109)



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Genereic Pin class for use with MicroPython boards"""
from adafruit_blinka import Enum


class Pin(Enum):
    """
    Identifies an IO pin on the microcontroller.

    They are fixed by the hardware so they cannot be constructed on demand. Instead, use board or
    microcontroller.pin to reference the desired pin.
    """

    def __init__(self, pin_id):
        """Identifier for pin, referencing platform-specific pin id"""
        self.id = pin_id

    def __repr__(self):
        # pylint: disable=import-outside-toplevel, cyclic-import
        import board
        import microcontroller.pin

        for key in dir(board):
            if getattr(board, key) is self:
                return "board.{}".format(key)
        # pylint: enable=import-outside-toplevel, cyclic-import

        for key in dir(microcontroller.pin):
            if getattr(microcontroller.pin, key) is self:
                return "microcontroller.pin.{}".format(key)
        return repr(self)



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""
`microcontroller` - Pin references and cpu functionality
========================================================

* Author(s): Melissa LeBlanc-Williams
"""

import sys
import time

from adafruit_platformdetect.constants import chips as ap_chip
from adafruit_blinka.agnostic import board_id, chip_id
from microcontroller import pin  # pylint: disable=unused-import
from microcontroller.pin import Pin  # pylint: disable=unused-import


def delay_us(delay):
    """Sleep for delay usecs."""
    time.sleep(delay / 1e6)


# We intentionally are patching into this namespace so skip the wildcard check.
# pylint: disable=unused-wildcard-import,wildcard-import,ungrouped-imports

if chip_id == ap_chip.ESP8266:
    from adafruit_blinka.microcontroller.esp8266 import *
elif chip_id == ap_chip.STM32F405:
    from adafruit_blinka.microcontroller.stm32.stm32f405 import *
elif chip_id == ap_chip.RP2040:
    from adafruit_blinka.microcontroller.rp2040 import *
elif chip_id == ap_chip.BCM2XXX:
    if board_id in [
        "RASPBERRY_PI_4B",
        "RASPBERRY_PI_400",
        "RASPBERRY_PI_CM4",
    ]:
        from adafruit_blinka.microcontroller.bcm2711 import *
    else:
        from adafruit_blinka.microcontroller.bcm283x import *
elif chip_id == ap_chip.DRA74X:
    from adafruit_blinka.microcontroller.dra74x import *
elif chip_id == ap_chip.AM33XX:
    from adafruit_blinka.microcontroller.am335x import *
elif chip_id == ap_chip.JH71x0:
    from adafruit_blinka.microcontroller.starfive.JH71x0 import *
elif chip_id == ap_chip.SUN8I:
    from adafruit_blinka.microcontroller.allwinner.h3 import *
elif chip_id == ap_chip.H5:
    from adafruit_blinka.microcontroller.allwinner.h5 import *
elif chip_id == ap_chip.H6:
    from adafruit_blinka.microcontroller.allwinner.h6 import *
elif chip_id == ap_chip.H616:
    from adafruit_blinka.microcontroller.allwinner.h616 import *
elif chip_id == ap_chip.SAMA5:
    from adafruit_blinka.microcontroller.sama5 import *
elif chip_id == ap_chip.T210:
    from adafruit_blinka.microcontroller.tegra.t210 import *
elif chip_id == ap_chip.T186:
    from adafruit_blinka.microcontroller.tegra.t186 import *
elif chip_id == ap_chip.T194:
    from adafruit_blinka.microcontroller.tegra.t194 import *
elif chip_id == ap_chip.S905:
    from adafruit_blinka.microcontroller.amlogic.s905 import *
elif chip_id == ap_chip.S905X3:
    from adafruit_blinka.microcontroller.amlogic.s905x3 import *
elif chip_id == ap_chip.S922X:
    from adafruit_blinka.microcontroller.amlogic.s922x import *
elif chip_id == ap_chip.A311D:
    from adafruit_blinka.microcontroller.amlogic.a311d import *
elif chip_id == ap_chip.EXYNOS5422:
    from adafruit_blinka.microcontroller.samsung.exynos5422 import *
elif chip_id == ap_chip.APQ8016:
    from adafruit_blinka.microcontroller.snapdragon.apq8016 import *
elif chip_id == ap_chip.A64:
    from adafruit_blinka.microcontroller.allwinner.a64 import *
elif chip_id == ap_chip.A33:
    from adafruit_blinka.microcontroller.allwinner.a33 import *
elif chip_id == ap_chip.RK3308:
    from adafruit_blinka.microcontroller.rockchip.rk3308 import *
elif chip_id == ap_chip.RK3399:
    from adafruit_blinka.microcontroller.rockchip.rk3399 import *
elif chip_id == ap_chip.RK3328:
    from adafruit_blinka.microcontroller.rockchip.rk3328 import *
elif chip_id == ap_chip.H3:
    from adafruit_blinka.microcontroller.allwinner.h3 import *
elif chip_id == ap_chip.H5:
    from adafruit_blinka.microcontroller.allwinner.h5 import *
elif chip_id == ap_chip.IMX8MX:
    from adafruit_blinka.microcontroller.nxp_imx8m import *
elif chip_id == ap_chip.IMX6ULL:
    from adafruit_blinka.microcontroller.nxp_imx6ull import *
elif chip_id == ap_chip.HFU540:
    from adafruit_blinka.microcontroller.hfu540 import *
elif chip_id == ap_chip.BINHO:
    from adafruit_blinka.microcontroller.nova import *
elif chip_id == ap_chip.LPC4330:
    from adafruit_blinka.microcontroller.nxp_lpc4330 import *
elif chip_id == ap_chip.MCP2221:
    from adafruit_blinka.microcontroller.mcp2221 import *
elif chip_id == ap_chip.MIPS24KC:
    from adafruit_blinka.microcontroller.atheros.ar9331 import *
elif chip_id == ap_chip.MIPS24KEC:
    from adafruit_blinka.microcontroller.mips24kec import *
elif chip_id == ap_chip.FT232H:
    from adafruit_blinka.microcontroller.ftdi_mpsse.ft232h import *
elif chip_id == ap_chip.FT2232H:
    from adafruit_blinka.microcontroller.ftdi_mpsse.ft2232h import *
elif chip_id == ap_chip.PENTIUM_N3710:
    from adafruit_blinka.microcontroller.pentium.n3710 import *
elif chip_id == ap_chip.STM32MP157:
    from adafruit_blinka.microcontroller.stm32.stm32mp157 import *
elif chip_id == ap_chip.MT8167:
    from adafruit_blinka.microcontroller.mt8167 import *
elif chip_id == ap_chip.RP2040_U2IF:
    from adafruit_blinka.microcontroller.rp2040_u2if import *
elif chip_id == ap_chip.GENERIC_X86:
    print("WARNING: GENERIC_X86 is not fully supported. Some features may not work.")
elif "sphinx" in sys.modules:
    pass
else:
    raise NotImplementedError("Microcontroller not supported:", chip_id)



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Raspberry Pi Pico pin names"""

from adafruit_blinka.microcontroller.rp2040 import pin

GP0 = pin.GP0
GP1 = pin.GP1
GP2 = pin.GP2
GP3 = pin.GP3
GP4 = pin.GP4
GP5 = pin.GP5
GP6 = pin.GP6
GP7 = pin.GP7
GP8 = pin.GP8
GP9 = pin.GP9
GP10 = pin.GP10
GP11 = pin.GP11
GP12 = pin.GP12
GP13 = pin.GP13
GP14 = pin.GP14
GP15 = pin.GP15
GP16 = pin.GP16
GP17 = pin.GP17
GP18 = pin.GP18
GP19 = pin.GP19
GP20 = pin.GP20
GP21 = pin.GP21
GP22 = pin.GP22
GP23 = pin.GP23
GP24 = pin.GP24
GP25 = pin.GP25
GP26 = pin.GP26
GP27 = pin.GP27
GP28 = pin.GP28
LED = GP25
SMPS_MODE = GP23
VBUS_SENSE = GP24
A0 = GP26
A1 = GP27
A2 = GP28
A3 = pin.GP29
VOLTAGE_MONITOR = pin.GP29



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Pin definitions for the MicroChip MCP2221"""
from adafruit_blinka.microcontroller.mcp2221 import pin

G0 = pin.G0
G1 = pin.G1
G2 = pin.G2
G3 = pin.G3

SCL = pin.SCL
SDA = pin.SDA



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Return the current version"""
VERSION = (1, 0, 0)

__version__ = ".".join(map(str, VERSION))



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""SPI Class for Generic MicroPython"""
from machine import SPI as _SPI

# pylint: disable=protected-access, no-self-use
class SPI:
    """SPI Class for Generic MicroPython"""

    MSB = _SPI.MSB
    LSB = _SPI.LSB

    def __init__(self, portId, baudrate=100000):
        self._frequency = baudrate
        self._spi = _SPI(portId)

    # pylint: disable=too-many-arguments,unused-argument
    def init(
        self,
        baudrate=1000000,
        polarity=0,
        phase=0,
        bits=8,
        firstbit=_SPI.MSB,
    ):
        """Initialize the Port"""
        self._frequency = baudrate
        self._spi.init(
            baudrate=baudrate,
            polarity=polarity,
            phase=phase,
            bits=bits,
            firstbit=firstbit,
            mode=_SPI.MASTER,
        )

    # pylint: enable=too-many-arguments

    @property
    def frequency(self):
        """Return the current frequency"""
        return self._frequency

    def write(self, buf, start=0, end=None):
        """Write data from the buffer to SPI"""
        self._spi.write(buf)

    def readinto(self, buf, start=0, end=None, write_value=0):
        """Read data from SPI and into the buffer"""
        self._spi.readinto(buf)

    # pylint: disable=too-many-arguments
    def write_readinto(
        self, buffer_out, buffer_in, out_start=0, out_end=None, in_start=0, in_end=None
    ):
        """Perform a half-duplex write from buffer_out and then
        read data into buffer_in
        """
        self._spi.write_readinto(
            buffer_out,
            buffer_in,
            out_start=out_start,
            out_end=out_end,
            in_start=in_start,
            in_end=in_end,
        )

    # pylint: enable=too-many-arguments



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
import time
import board
import busio

spi = busio.SPI(board.SCLK, board.MOSI, board.MISO)
while not spi.try_lock():
    pass
spi.configure(baudrate=16000000)
spi.unlock()

while True:
    spi.write(bytes(range(64)))
    time.sleep(0.1)



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Pin definitions for the Khadas VIM3."""

from adafruit_blinka.microcontroller.amlogic.a311d import pin

GPIOAO_0 = pin.GPIO496
GPIOAO_1 = pin.GPIO497
GPIOAO_2 = pin.GPIO498
GPIOAO_3 = pin.GPIO499
GPIOAO_4 = pin.GPIO500
GPIOAO_5 = pin.GPIO501
GPIOAO_6 = pin.GPIO502
GPIOAO_7 = pin.GPIO503
GPIOAO_8 = pin.GPIO504
GPIOAO_9 = pin.GPIO505
GPIOAO_10 = pin.GPIO506
GPIOAO_11 = pin.GPIO507

GPIOE_0 = pin.GPIO508
GPIOE_1 = pin.GPIO509
GPIOE_2 = pin.GPIO510

GPIOZ_0 = pin.GPIO427
GPIOZ_1 = pin.GPIO428
GPIOZ_2 = pin.GPIO429
GPIOZ_3 = pin.GPIO430
GPIOZ_4 = pin.GPIO431
GPIOZ_5 = pin.GPIO432
GPIOZ_6 = pin.GPIO433
GPIOZ_7 = pin.GPIO434
GPIOZ_8 = pin.GPIO435
GPIOZ_9 = pin.GPIO436
GPIOZ_10 = pin.GPIO437
GPIOZ_11 = pin.GPIO438
GPIOZ_12 = pin.GPIO439
GPIOZ_13 = pin.GPIO440
GPIOZ_14 = pin.GPIO441
GPIOZ_15 = pin.GPIO442

GPIOH_0 = pin.GPIO443
GPIOH_1 = pin.GPIO444
GPIOH_2 = pin.GPIO445
GPIOH_3 = pin.GPIO446
GPIOH_4 = pin.GPIO447
GPIOH_5 = pin.GPIO448
GPIOH_6 = pin.GPIO449
GPIOH_7 = pin.GPIO450
GPIOH_8 = pin.GPIO451

BOOT_0 = pin.GPIO452
BOOT_1 = pin.GPIO453
BOOT_2 = pin.GPIO454
BOOT_3 = pin.GPIO455
BOOT_4 = pin.GPIO456
BOOT_5 = pin.GPIO457
BOOT_6 = pin.GPIO458
BOOT_7 = pin.GPIO459
BOOT_8 = pin.GPIO460
BOOT_9 = pin.GPIO461
BOOT_10 = pin.GPIO462
BOOT_11 = pin.GPIO463
BOOT_12 = pin.GPIO464
BOOT_13 = pin.GPIO465
BOOT_14 = pin.GPIO466
BOOT_15 = pin.GPIO467

GPIOC_0 = pin.GPIO468
GPIOC_1 = pin.GPIO469
GPIOC_2 = pin.GPIO470
GPIOC_3 = pin.GPIO471
GPIOC_4 = pin.GPIO472
GPIOC_5 = pin.GPIO473
GPIOC_6 = pin.GPIO474
GPIOC_7 = pin.GPIO475

GPIOA_0 = pin.GPIO460
GPIOA_1 = pin.GPIO461
GPIOA_2 = pin.GPIO462
GPIOA_3 = pin.GPIO463
GPIOA_4 = pin.GPIO464
GPIOA_5 = pin.GPIO465
GPIOA_6 = pin.GPIO466
GPIOA_7 = pin.GPIO467
GPIOA_8 = pin.GPIO468
GPIOA_9 = pin.GPIO469
GPIOA_10 = pin.GPIO470
GPIOA_11 = pin.GPIO471
GPIOA_12 = pin.GPIO472
GPIOA_13 = pin.GPIO473
GPIOA_14 = pin.GPIO474
GPIOA_15 = pin.GPIO475

GPIOX_0 = pin.GPIO476
GPIOX_1 = pin.GPIO477
GPIOX_2 = pin.GPIO478
GPIOX_3 = pin.GPIO479
GPIOX_4 = pin.GPIO480
GPIOX_5 = pin.GPIO481
GPIOX_6 = pin.GPIO482
GPIOX_7 = pin.GPIO483
GPIOX_8 = pin.GPIO484
GPIOX_9 = pin.GPIO485
GPIOX_10 = pin.GPIO486
GPIOX_11 = pin.GPIO487
GPIOX_12 = pin.GPIO488
GPIOX_13 = pin.GPIO489
GPIOX_14 = pin.GPIO490
GPIOX_15 = pin.GPIO491
GPIOX_16 = pin.GPIO492
GPIOX_17 = pin.GPIO493
GPIOX_18 = pin.GPIO494
GPIOX_19 = pin.GPIO495

SCL = pin.GPIOX_18
SDA = pin.GPIOX_17

SCLK = pin.SPI0_SCLK
MCLK0 = pin.SPI0_MCLK0
MISO = pin.SPI0_SDO
MOSI = pin.SPI0_SDI

D0 = GPIOAO_10  # PIN_13
D1 = GPIOH_6  # PIN_15
D2 = GPIOH_7  # PIN_16
D3 = GPIOAO_1  # PIN_18
D4 = GPIOAO_2  # PIN_19
D5 = GPIOA_15  # PIN_22
D6 = GPIOA_14  # PIN_23
D7 = GPIOAO_2  # PIN_25
D8 = GPIOAO_3  # PIN_26
D9 = GPIOA_1  # PIN_29
D10 = GPIOA_0  # PIN_30
D11 = GPIOA_3  # PIN_31
D12 = GPIOA_2  # PIN_32
D13 = GPIOA_4  # PIN_33
D14 = GPIOH_5  # PIN_35
D15 = GPIOH_4  # PIN_37
D16 = GPIOZ_15  # PIN_39



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Pin definitions for the GreatFET One"""
from adafruit_blinka.microcontroller.nxp_lpc4330 import pin

J1_P3 = pin.J1_P3
J1_P4 = pin.J1_P4
J1_P5 = pin.J1_P5
J1_P6 = pin.J1_P6
J1_P7 = pin.J1_P7
J1_P8 = pin.J1_P8
J1_P9 = pin.J1_P9
J1_P10 = pin.J1_P10
J1_P12 = pin.J1_P12
J1_P13 = pin.J1_P13
J1_P14 = pin.J1_P14
J1_P15 = pin.J1_P15
J1_P16 = pin.J1_P16
J1_P17 = pin.J1_P17
J1_P18 = pin.J1_P18
J1_P19 = pin.J1_P19
J1_P20 = pin.J1_P20
J1_P21 = pin.J1_P21
J1_P22 = pin.J1_P22
J1_P23 = pin.J1_P23
J1_P24 = pin.J1_P24
J1_P25 = pin.J1_P25
J1_P26 = pin.J1_P26
J1_P27 = pin.J1_P27
J1_P28 = pin.J1_P28
J1_P29 = pin.J1_P29
J1_P30 = pin.J1_P30
J1_P31 = pin.J1_P31
J1_P32 = pin.J1_P32
J1_P33 = pin.J1_P33
J1_P34 = pin.J1_P34
J1_P35 = pin.J1_P35
J1_P37 = pin.J1_P37
J1_P39 = pin.J1_P39
J1_P40 = pin.J1_P40

# J2 Header Pins
J2_P3 = pin.J2_P3
J2_P4 = pin.J2_P4
J2_P5 = pin.J2_P5
J2_P6 = pin.J2_P6
J2_P7 = pin.J2_P7
J2_P8 = pin.J2_P8
J2_P9 = pin.J2_P9
J2_P10 = pin.J2_P10
J2_P13 = pin.J2_P13
J2_P14 = pin.J2_P14
J2_P15 = pin.J2_P15
J2_P16 = pin.J2_P16
J2_P18 = pin.J2_P18
J2_P19 = pin.J2_P19
J2_P20 = pin.J2_P20
J2_P22 = pin.J2_P22
J2_P23 = pin.J2_P23
J2_P24 = pin.J2_P24
J2_P25 = pin.J2_P25
J2_P27 = pin.J2_P27
J2_P28 = pin.J2_P28
J2_P29 = pin.J2_P29
J2_P30 = pin.J2_P30
J2_P31 = pin.J2_P31
J2_P33 = pin.J2_P33
J2_P34 = pin.J2_P34
J2_P35 = pin.J2_P35
J2_P36 = pin.J2_P36
J2_P37 = pin.J2_P37
J2_P38 = pin.J2_P38

# Bonus Row Pins
J7_P2 = pin.J7_P2
J7_P3 = pin.J7_P3
J7_P6 = pin.J7_P6
J7_P7 = pin.J7_P7
J7_P8 = pin.J7_P8
J7_P13 = pin.J7_P13
J7_P14 = pin.J7_P14
J7_P15 = pin.J7_P15
J7_P16 = pin.J7_P16
J7_P17 = pin.J7_P17
J7_P18 = pin.J7_P18

SDA = pin.SDA
SCL = pin.SCL

SCK = pin.SCK
SCLK = SCK
MOSI = pin.MOSI
MISO = pin.MISO

TX = pin.TX
RX = pin.RX



# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Pin definitions for the Octavo OSD32MP1 RED board."""

from adafruit_blinka.microcontroller.stm32.stm32mp157 import pin

# RPi header JP20
D0 = pin.PF15
D1 = pin.PD12
D2 = pin.PA12
D3 = pin.PA11
D4 = pin.PI11
D5 = pin.PG2
D6 = pin.PH11
D7 = pin.PF13
D8 = pin.PF6
D9 = pin.PF8
D10 = pin.PF9
D11 = pin.PF7
D12 = pin.PD13
D13 = pin.PB5
D14 = pin.PB10
D15 = pin.PB12
D16 = pin.PB13
D17 = pin.PG8
D18 = pin.PI5
D19 = pin.PI7
D20 = pin.PI6
D21 = pin.PE11
D22 = pin.PG15
D23 = pin.PF1
D24 = pin.PF0
D25 = pin.PF4
D26 = pin.PF5
D27 = pin.PD7

I2C5_SDA = SDA = D2
I2C5_SCL = SCL = D3

I2C1_SDA = ID_SD = D0
I2C1_SCL = ID_SC = D1

SPI5_SCLK = SCLK = SCK = D11
SPI5_MOSI = MOSI = D10
SPI5_MISO = MISO = D9
SPI5_NSS = CS = CE0 = D8

PI_GPIO7 = CE1 = D7

UART_TX = D14
UART_RX = D15

# Console UART
UART4_TX = pin.PG11
UART4_RX = pin.PB2
